# pip install matplotlib PySide6
import sys
from PySide6.QtWidgets import QApplication, QMainWindow, QWidget, QVBoxLayout, QPushButton
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure


class MplCanvas(FigureCanvas):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        self.ax = self.fig.add_subplot(111)
        super().__init__(self.fig)
        self.setParent(parent)


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("PySide6 + Matplotlib")

        central = QWidget()
        layout = QVBoxLayout(central)

        self.canvas = MplCanvas(self, width=6, height=4, dpi=100)
        layout.addWidget(self.canvas)

        btn = QPushButton("Plot")
        btn.clicked.connect(self.plot_example)
        layout.addWidget(btn)

        self.setCentralWidget(central)
        # self.plot_example()

    def plot_example(self):
        ax = self.canvas.ax
        ax.clear()
        x = [0, 1, 2, 3, 4]
        y = [0, 1, 4, 9, 16]
        ax.plot(x, y)
        ax.set_title("Embedded Matplotlib")
        ax.figure.tight_layout()
        self.canvas.draw()          # draw() for immediate update
        # self.canvas.draw_idle()   # nicer if you update often


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = MainWindow()
    w.resize(800, 600)
    w.show()
    sys.exit(app.exec())



import os
import shutil
import zipfile


DATA_DIR = 'M:/data/lauracools/testt4selectiontotalseg/original'
DATA_UNZIPPED_DIR = 'M:/data/lauracools/testt4selectiontotalseg/original_unzipped'
DATA_UNZIPPED_NODOTS_DIR = 'M:/data/lauracools/testt4selectiontotalseg/original_unzipped_nodots'
os.makedirs(DATA_UNZIPPED_DIR, exist_ok=True)
os.makedirs(DATA_UNZIPPED_NODOTS_DIR, exist_ok=True)


for f in os.listdir(DATA_DIR):
    f_path = os.path.join(DATA_DIR, f)
    if f.endswith('.zip'):

        unzipped_dir_path = os.path.join(DATA_UNZIPPED_DIR, f[:-4])
        os.makedirs(unzipped_dir_path, exist_ok=True)
        
        with zipfile.ZipFile(f_path, 'r') as zf:
            zf.extractall(unzipped_dir_path)

            unzipped_nodots_dir_path = os.path.join(DATA_UNZIPPED_NODOTS_DIR, f[:-4])
            os.makedirs(unzipped_nodots_dir_path, exist_ok=True)
            
            for dicom_file in os.listdir(unzipped_dir_path):
                dicom_file_path = os.path.join(unzipped_dir_path, dicom_file)
                if dicom_file.startswith('.'):
                    shutil.copy(dicom_file_path, os.path.join(unzipped_nodots_dir_path, dicom_file[1:]))
                else:
                    shutil.copy(dicom_file_path, unzipped_nodots_dir_path)
            print(f'Extracted {f}')


import sys
from PySide6.QtWidgets import QApplication, QMainWindow, QFrame, QVBoxLayout, QWidget
from PySide6.QtCore import Qt

import vtk
from vtkmodules.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor


class VTKGraphWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        # Qt layout
        layout = QVBoxLayout()
        self.setLayout(layout)

        # VTK interactor widget
        self.vtkWidget = QVTKRenderWindowInteractor(self)
        layout.addWidget(self.vtkWidget)

        # Build a simple graph
        g = vtk.vtkMutableUndirectedGraph()
        v1 = g.AddVertex()
        v2 = g.AddVertex()
        v3 = g.AddVertex()
        v4 = g.AddVertex()
        g.AddEdge(v1, v2)
        g.AddEdge(v2, v3)
        g.AddEdge(v3, v4)
        g.AddEdge(v4, v1)
        g.AddEdge(v1, v3)  # cross edge

        # Layout & view
        self.view = vtk.vtkGraphLayoutView()
        self.view.AddRepresentationFromInput(g)
        self.view.SetLayoutStrategy("Force Directed")

        # Use the same render window
        self.view.SetRenderWindow(self.vtkWidget.GetRenderWindow())

        # Start interactor
        self.interactor = self.vtkWidget.GetRenderWindow().GetInteractor()
        self.view.ResetCamera()


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("VTK Infovis Graph in PySide6")
        self.resize(800, 600)

        frame = QFrame()
        layout = QVBoxLayout()
        frame.setLayout(layout)

        # Add VTK graph widget
        self.graphWidget = VTKGraphWidget(self)
        layout.addWidget(self.graphWidget)

        self.setCentralWidget(frame)


if __name__ == "__main__":
    app = QApplication(sys.argv)

    window = MainWindow()
    window.show()

    # Start VTK interactor loop
    window.graphWidget.interactor.Initialize()
    window.graphWidget.interactor.Start()

    sys.exit(app.exec())



from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qtagg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure


class MatplotlibCanvas(FigureCanvas):
    def __init__(self, parent, nrows=1, ncols=1, width=6, height=4, dpi=100):
        self._figure = Figure(figsize=(width, height), dpi=dpi)
        self._axes = self._figure.subplots(nrows, ncols)
        super(MatplotlibCanvas, self).__init__(self._figure)
        self.setParent(parent)
        self._nrows = nrows
        self._ncols = ncols
        self._navtoolbar = None

    def axes(self, idx=0, idy=0):
        assert idx<self._nrows and idy<self._ncols
        if idx>0 and idy>0:
            return self._axes[idx, idy]
        elif idx>0:
            return self._axes[idx]
        else:
            return self._axes
        
    def navtoolbar(self):
        if not self._navtoolbar:
            self._navtoolbar = NavigationToolbar(self, self.parent())
        return self._navtoolbar


import os
from PySide6.QtWidgets import QWidget, QVBoxLayout, QMessageBox, QScrollArea, QGridLayout
from PySide6.QtCore import Qt, Signal, QSize
from mosamatic2.ui.widgets.panels.visualizations.sliceselectionvisualization.slicetile import SliceTile
from mosamatic2.ui.widgets.panels.visualizations.sliceselectionvisualization.slicetile import SliceItem

# https://chatgpt.com/c/69494292-b8fc-8327-863f-438838247bd4


class SliceSelectionViewer(QWidget):
    selection_changed = Signal()

    def __init__(self, columns: int = 4, tile_size: QSize = QSize(220, 220), parent=None):
        super(SliceSelectionViewer, self).__init__()
        self._columns = columns
        self._tile_size = tile_size
        self._tiles: list[SliceTile] = []
        self._scroll_area = QScrollArea()
        self._scroll_area.setWidgetResizable(True)
        self._container = QWidget()
        self._grid = QGridLayout(self._container)
        self._grid.setContentsMargins(10, 10, 10, 10)
        self._grid.setSpacing(10)
        self._scroll_area.setWidget(self._container)
        self._root = QVBoxLayout(self)
        self._root.addWidget(self._scroll_area)

    def clear(self):
        # remove widgets from layout
        while self._grid.count():
            item = self._grid.takeAt(0)
            w = item.widget()
            if w:
                w.setParent(None)
        self._tiles.clear()

    def load_images(self, images_dir):
        images = []
        for f in os.listdir(images_dir):
            if f.endswith('_sagittal.png'):
                f_path = os.path.join(images_dir, f)
                images.append(f_path)
        self.clear()
        for i, p in enumerate(images):
            tile = SliceTile(SliceItem(p), self._tile_size)
            tile.toggled.connect(self._on_tile_toggled)
            self._tiles.append(tile)
            r, c = divmod(i, self._columns)
            self._grid.addWidget(tile, r, c)
        # Optional: keep grid left-aligned by adding stretch
        self._grid.setColumnStretch(self._columns, 1)

    def selected_paths(self) -> list[str]:
        return [t.item.path for t in self._tiles if t.is_checked()]
    
    def set_all_checked(self, checked: bool):
        for t in self._tiles:
            t.set_checked(checked)

    def _on_tile_toggled(self, _path: str, _checked: bool):
        self.selection_changed.emit()


import os
import shutil
import numpy as np
import SimpleITK as sitk
from mosamatic2.core.tasks.task import Task
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.core.utils import (
    is_dicom, 
    convert_numpy_array_to_png_image, 
    convert_dicom_to_png_image,
    AlbertaColorMap,
)

LOG = LogManager()


class SegmentationNumpy2NiftiTask(Task):
    INPUTS = ['images', 'segmentations']
    PARAMS = []

    def __init__(self, inputs, params, output, overwrite):
        super(SegmentationNumpy2NiftiTask, self).__init__(inputs, params, output, overwrite)

    def load_images(self):
        images = []
        for f in os.listdir(self.input('images')):
            f_path = os.path.join(self.input('images'), f)
            if is_dicom(f_path):
                images.append(f_path)
        return images

    def load_segmentations(self):
        segmentations = []
        for f in os.listdir(self.input('segmentations')):
            if f.endswith('.seg.npy'):
                f_path = os.path.join(self.input('segmentations'), f)
                segmentations.append(f_path)
        return segmentations
    
    def create_pairs_of_images_and_segmentations(self, images, segmentations):
        pairs = []
        for segmentation in segmentations:
            for image in images:
                image_name = os.path.split(image)[1]
                if image_name in os.path.split(segmentation)[1]:
                    pairs.append((image, segmentation))
        return pairs
    
    def load_segmentation_as_narray(self, segmentation):
        narray = np.load(segmentation)
        return narray
    
    def create_png_from_dicom(self, file_path):
        convert_dicom_to_png_image(file_path, self.output())

    def create_png_from_array(self, data, file_path):
        png_file_name = os.path.split(file_path)[1] + '.png'
        convert_numpy_array_to_png_image(
            data, 
            self.output(),
            AlbertaColorMap(), 
            png_file_name,
            fig_width=10, fig_height=10,
        )


    def run(self):
        images = self.load_images()
        segmentations = self.load_segmentations()        
        image_and_segmentation_pairs = self.create_pairs_of_images_and_segmentations(images, segmentations)
        nr_steps = len(image_and_segmentation_pairs)
        for step in range(nr_steps):
            pair = image_and_segmentation_pairs[step]
            # Load segmentation
            segmentation_narray = self.load_segmentation_as_narray(pair[1])
            segmentation_narray = segmentation_narray.astype(np.uint16)
            segmentation_narray3d = segmentation_narray[None, ...]
            # Load DICOM
            image_itk = sitk.ReadImage(pair[0])
            segmentation_itk = sitk.GetImageFromArray(segmentation_narray3d)
            segmentation_itk.CopyInformation(image_itk)
            segmentation_itk_name = os.path.split(pair[1])[1] + '.nii.gz'
            segmentation_itk_path = os.path.join(self.output(), segmentation_itk_name)
            sitk.WriteImage(segmentation_itk, segmentation_itk_path, useCompression=True)
            # Copy NIFTI file to NumPy segmentation directory as well
            seg_dir = os.path.split(pair[1])[0]
            shutil.copy(segmentation_itk_path, seg_dir)
            # Create PNG images
            self.create_png_from_array(segmentation_narray, os.path.join(self.output(), segmentation_itk_name))
            self.create_png_from_dicom(pair[0])
            self.set_progress(step, nr_steps)


import os
import shutil


DATA_DIR = 'M:/data/lauracools/testt4selectiontotalseg/original_unzipped\STS001_CT-thorax'
DATA_OUTPUT_DIR = 'M:/data/lauracools/testt4selectiontotalseg/original_unzipped_nodots\STS001_CT-thorax'
os.makedirs(DATA_OUTPUT_DIR, exist_ok=True)

for f in os.listdir(DATA_DIR):
    f_path = os.path.join(DATA_DIR, f)
    if f.startswith('.'):
        shutil.copy(f_path, os.path.join(DATA_OUTPUT_DIR, f[1:]))
    else:
        shutil.copy(f_path, DATA_OUTPUT_DIR)
    print(f_path)


import vtk
import sys
import pydicom
import numpy as np

from vtkmodules.util import numpy_support

DICOM_FILE_NAME = '04RAND092.dcm'
DICOM_FILE_PATH = f'/Users/ralph/Library/CloudStorage/GoogleDrive-ralph.brecheisen@gmail.com/My Drive/data/mosamatic2/data/l3_edema/{DICOM_FILE_NAME}'
SEGMENTATION_FILE_NAME = '04RAND092.dcm.seg.npy'
SEGMENTATION_FILE_PATH = f'/Users/ralph/Library/CloudStorage/GoogleDrive-ralph.brecheisen@gmail.com/My Drive/data/mosamatic2/data/l3_edema/{SEGMENTATION_FILE_NAME}'


def numpy_to_vtk_image(arr2d, vtk_scalar_type):
    h, w = arr2d.shape
    vtk_arr = numpy_support.numpy_to_vtk(
        num_array=np.ascontiguousarray(arr2d).ravel(order="C"),
        deep=True,
        array_type=vtk_scalar_type,
    )
    img = vtk.vtkImageData()
    img.SetDimensions(w, h, 1)
    img.GetPointData().SetScalars(vtk_arr)
    return img


def update_vtk_image_from_numpy(vtk_img, arr2d, vtk_type):
    h, w = arr2d.shape
    flat = np.ascontiguousarray(arr2d).ravel(order="C")
    vtk_arr = numpy_support.numpy_to_vtk(flat, deep=True, array_type=vtk_type)
    vtk_arr.SetName("scalars")
    vtk_img.SetDimensions(w, h, 1)
    vtk_img.GetPointData().SetScalars(vtk_arr)
    vtk_img.Modified()
    return vtk_img


def make_binary_rgba_overlay(binary_mask, rgba=(1,0,0,0.35)):
    """binary_mask is (H,W) uint8 {0,1}; returns vtkImageData RGBA."""
    vtk_mask = vtk.vtkImageData()
    update_vtk_image_from_numpy(vtk_mask, binary_mask.astype(np.uint8), vtk.VTK_UNSIGNED_CHAR)

    lut = vtk.vtkLookupTable()
    lut.SetNumberOfTableValues(2)
    lut.SetRange(0, 1)
    lut.Build()
    lut.SetTableValue(0, 0, 0, 0, 0)  # background transparent
    lut.SetTableValue(1, *rgba)

    mapper = vtk.vtkImageMapToColors()
    mapper.SetInputData(vtk_mask)
    mapper.SetLookupTable(lut)
    mapper.SetOutputFormatToRGBA()
    mapper.Update()

    # Return both: vtk_mask (so you can update it later) and mapper output port
    return vtk_mask, mapper


def get_lut(mask):
    lut = vtk.vtkLookupTable()
    max_label = int(mask.max())
    lut.SetNumberOfTableValues(max_label + 1)
    lut.SetRange(0, max_label)
    lut.Build()
    for i in range(max_label + 1):
        lut.SetTableValue(i, 0.0, 0.0, 0.0, 0.0)
    lut.SetTableValue(0, 0.0, 0.0, 0.0, 0.0)
    lut.SetTableValue(1, 1.0, 0.0, 0.0, 1.0)
    lut.SetTableValue(5, 1.0, 1.0, 0.0, 1.0)
    lut.SetTableValue(7, 0.0, 1.0, 1.0, 1.0)
    return lut


def main():

    L = 1
    alpha = 1.0
    threshold = 30

    # Load L3 image
    print('Loading image')
    p = pydicom.dcmread(DICOM_FILE_PATH)
    image = p.pixel_array.astype(np.float32)
    image = image * float(getattr(p, "RescaleSlope", 1.0)) + float(getattr(p, "RescaleIntercept", 0.0))
    image = np.flipud(image)
    # vtk_image = numpy_to_vtk_image(image, vtk.VTK_FLOAT)

    # Load segmentation mask
    print('Loading segmentation mask')
    mask = np.load(SEGMENTATION_FILE_PATH).astype(np.uint8)
    mask = np.flipud(mask)
    label_mask = (mask == L)
    hi = (label_mask & (image >= threshold)).astype(np.uint8)
    lo = (label_mask & (image <  threshold)).astype(np.uint8)
    vtk_hi_mask, hi_rgba = make_binary_rgba_overlay(hi, rgba=(1, 0, 1, alpha))
    vtk_lo_mask, lo_rgba = make_binary_rgba_overlay(lo, rgba=(1, 1, 0, alpha))
    # vtk_mask = numpy_to_vtk_image(mask, vtk.VTK_UNSIGNED_CHAR)

    # # Apply window/level to image
    # wl = vtk.vtkImageMapToWindowLevelColors()
    # wl.SetInputData(vtk_image)
    # wl.SetWindow(400.0)
    # wl.SetLevel(50.0)
    # wl.Update()

    # # Apply colors to mask
    # mask_rgba = vtk.vtkImageMapToColors()
    # mask_rgba.SetInputData(vtk_mask)
    # mask_rgba.SetLookupTable(get_lut(mask))
    # mask_rgba.SetOutputFormatToRGBA()
    # mask_rgba.Update()

    hi_actor = vtk.vtkImageActor()
    hi_actor.GetMapper().SetInputConnection(hi_rgba.GetOutputPort())
    hi_actor.PickableOff()

    lo_actor = vtk.vtkImageActor()
    lo_actor.GetMapper().SetInputConnection(lo_rgba.GetOutputPort())
    lo_actor.PickableOff()

    # # Create image actor
    # image_actor = vtk.vtkImageActor()
    # image_actor.GetMapper().SetInputConnection(wl.GetOutputPort())
    # image_actor.PickableOff()

    # # Create mask actor
    # mask_actor = vtk.vtkImageActor()
    # mask_actor.GetMapper().SetInputConnection(mask_rgba.GetOutputPort())

    # # Create blend between image and mask
    # blend = vtk.vtkImageBlend()
    # blend.AddInputConnection(wl.GetOutputPort())
    # blend.AddInputConnection(mask_rgba.GetOutputPort())
    # blend.SetOpacity(0, 1.0)  # base
    # blend.SetOpacity(1, 1.0)  # overlay (alpha comes from LUT)
    # blend.Update()

    # Create renderer
    ren = vtk.vtkRenderer()
    # ren.AddActor(image_actor)
    # ren.AddActor(mask_actor)
    ren.AddActor(hi_actor)
    ren.AddActor(lo_actor)
    ren.GetActiveCamera().ParallelProjectionOn()
    ren.ResetCamera()

    # Create render window
    renwin = vtk.vtkRenderWindow()
    renwin.AddRenderer(ren)
    renwin.SetSize(900, 900)

    # Create render window interactor
    iren = vtk.vtkRenderWindowInteractor()
    iren.SetInteractorStyle(vtk.vtkInteractorStyleImage())
    iren.SetRenderWindow(renwin)

    # # Create 2D image viewer
    # viewer = vtk.vtkImageViewer2()
    # viewer.SetInputConnection(blend.GetOutputPort())
    # viewer.SetupInteractor(iren)
    # viewer.GetRenderWindow().SetSize(900, 900)
    # viewer.Render()

    renwin.Render()
    iren.Start()


if __name__ == '__main__':
    main()


BASE_DIR = 'D:\\SoftwareDevelopment\\GitHub\\mosamatic2\\mosamatic2\\src\\experiments\\boamosamatic'

MDR_PROJECT_ID = 'P000000420'       # PORSCH trial
MDR_COLLECTION_ID = 'C000000002'    # PORSCH trial CT scans

LOCAL_L3_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\PORSCH\\L3'
LOCAL_ZIP_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\PORSCH\\ZIP'
LOCAL_UNZIPPED_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\PORSCH\\UNZIPPED'
LOCAL_SCAN_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\PORSCH\\SCANS'
LOCAL_SCAN_NIFTI_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\PORSCH\\SCANS_NIFTI'
LOCAL_SCAN_NIFTI_L3_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\PORSCH\\SCANS_NIFTI_L3'


import sys
import traceback
import faulthandler
import mosamatic2.constants as constants
from PySide6 import QtWidgets
from PySide6 import QtCore
from mosamatic2.ui.settings import Settings
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.ui.widgets.splashscreen import SplashScreen
# faulthandler.enable(all_threads=True)
LOG = LogManager()


def excepthook(exc_type, exc, tb):
    LOG.error(traceback.format_exc())
    # traceback.print_exception(exc_type, exc, tb)
# sys.excepthook = excepthook


def qt_message_handler(msg_type, ctx, msg):
    LOG.error(f"QT[{msg_type}]: {msg} ({ctx.file}:{ctx.line})")
# QtCore.qInstallMessageHandler(qt_message_handler)


def run_tests():
    return 'PASSED'


def main():
    settings = Settings()
    application_name = settings.get(constants.MOSAMATIC2_WINDOW_TITLE)
    QtWidgets.QApplication.setApplicationName(application_name)
    app = QtWidgets.QApplication(sys.argv)
    app.setApplicationName(application_name)
    splash = SplashScreen()
    splash.show()
    # raise SystemExit(app.exec())
    sys.exit(app.exec())
    # try:
    #     splash = SplashScreen()
    #     splash.show()
    #     # raise SystemExit(app.exec())
    #     sys.exit(app.exec())
    # except Exception as e:
    #     LOG.error(str(e))
    #     LOG.error(traceback.format_exc())
    #     sys.exit(1)

if __name__ == '__main__':
    main()


import os
import constants
from utils import (
    print_dictionary, 
    load_dictionary, 
    save_dictionary, 
    get_path, 
    get_parent_path,
    get_parent_name,
    is_number,
    get_webdav_client,
)

client = get_webdav_client()


def main():
    folder_mapping = load_dictionary('folder_mapping.json')
    image_file_paths = load_dictionary('images.json')
    for image_file_path in image_file_paths:
        # Get local parent directory name
        local_folder = get_parent_name(image_file_path)
        # Lookup remote directory name
        remote_folder = folder_mapping[local_folder]
        remote_folder_path = f'{constants.MDR_PROJECT_ID}/{constants.MDR_COLLECTION_ID}/{remote_folder}'
        # Build path to remote ZIP file
        remote_zip_file_name = os.path.split(image_file_path)[1].split('.')[0] + '.zip'
        remote_zip_file_path = f'{remote_folder_path}/{remote_zip_file_path}'
        # Download remote ZIP file
        # Unpack it ZIP file
        # Find path to correct CT scan
        # Copy CT scan to separate directory and save path
        # Delete ZIP file and unpacked ZIP directory
        # Store image file path and CT scan path in dictionary
        # Next image
        pass


if __name__ == '__main__':
    main()


# def load_image_file_paths():
#     image_file_paths = []
#     for d in os.listdir(constants.LOCAL_L3_BASE_DIR):
#         d_path = os.path.join(constants.LOCAL_L3_BASE_DIR, d)
#         if os.path.isdir(d_path):
#             for f in os.listdir(d_path):
#                 if is_number(f):
#                     image_file_paths.append(os.path.join(d_path, f))
#     save_dictionary(image_file_paths, 'images.json')
#     return image_file_paths


# def create_folder_l3_image_mapping(folder_mapping):
#     l3_image_mapping = {}
#     for local_folder_name, remote_folder_name in folder_mapping.items():
#         local_folder_path = os.path.join(constants.LOCAL_L3_BASE_DIR, local_folder_name)
#         l3_image_mapping[local_folder_name] = []
#         for file_name in os.listdir(local_folder_path):
#             if is_number(file_name):
#                 file_path = os.path.join(local_folder_path, file_name)
#                 l3_image_mapping[local_folder_name].append(file_path)
#     save_dictionary(l3_image_mapping, 'folder_l3_image_mapping.json')
#     return l3_image_mapping


# def download_zip_file_from_remote(file_name, remote_folder_name):
#     remote_folder_path = f'{constants.MDR_PROJECT_ID}/{constants.MDR_COLLECTION_ID}/{remote_folder_name}'
#     file_name_no_ext = file_name.split('.')[0]
#     zip_file_name = f'{file_name_no_ext}.zip'
#     zip_file_path = f'{remote_folder_path}/{zip_file_name}'
#     if CLIENT.check(zip_file_path):
#         print(f'ZIP file found')
#     return ''


# def create_l3_to_scan_mapping(folder_mapping):
#     l3_to_remote_scan_mapping = {}
#     for local_folder_name, remote_folder_name in folder_mapping.items():
#         local_folder_path = os.path.join(constants.LOCAL_L3_BASE_DIR, local_folder_name)
#         l3_to_remote_scan_mapping[local_folder_name] = {}
#         for file_name in os.listdir(local_folder_path):
#             if is_number(file_name):
#                 file_path = os.path.join(local_folder_path, file_name)
#                 zip_file_path = download_zip_file_from_remote(file_name, remote_folder_name)

#             l3_to_remote_scan_mapping[local_folder_name][file_path] = ''
#         # print_dictionary(l3_to_remote_scan_mapping[local_folder_name])
#     return l3_to_remote_scan_mapping



from mosamatic2.core.data.filedata import FileData
from mosamatic2.core.utils import (
    is_nifti,
    load_nifti,
)

class NiftiImage(FileData):
    def load(self):
        if self.path():
            if is_nifti(self.path()):
                self.set_object(load_nifti(self.path()))
                return True
        return False


import os
import pydicom
from collections import defaultdict, Counter

ROOT_DIR = 'D:\\Mosamatic\\Dixon\\AllFiles'


def scan_dicom_directory(root_dir):
    results = defaultdict(lambda: defaultdict(Counter))
    for dirpath, _, filenames in os.walk(root_dir):
        for filename in filenames:
            filepath = os.path.join(dirpath, filename)
            try:
                ds = pydicom.dcmread(filepath, stop_before_pixels=True, force=True)
                modality = getattr(ds, "Modality", "UNKNOWN")
                series_desc = getattr(ds, "SeriesDescription", "NoDescription")
                image_type = tuple(getattr(ds, "ImageType", []))
                if modality == "MR":
                    if ("DIXON" in series_desc.upper()) or any(
                        word in str(image_type).upper()
                        for word in ["DIXON", "WATER", "FAT", "IN_PHASE", "OPPOSED_PHASE"]
                    ):
                        modality = "DIXON"
                results[modality][series_desc][image_type] += 1
            except Exception:
                continue
    return results


def test_dicomdir_scan():
    dicom_summary = scan_dicom_directory(ROOT_DIR)
    print("\nDICOM summary:")
    for modality, series_dict in dicom_summary.items():
        print(f"\nModality: {modality}")
        total_modality_files = sum(sum(counter.values()) for counter in series_dict.values())
        print(f"  -> Total files: {total_modality_files}")
        for series, counter in series_dict.items():
            print(f"  Series: {series}")
            for img_type, count in counter.items():
                print(f"    ImageType: {img_type} -> {count} files")


import vtk
# https://pycad.medium.com/dicom-viewer-in-python-5679a675a560


class CustomInteractorStyle(vtk.vtkInteractorStyleImage):
    def __init__(self, image_viewer, status_actor):
        super().__init__()
        self.AddObserver('MouseWheelForwardEvent', self.move_slice_forward)
        self.AddObserver('MouseWheelBackwardEvent', self.move_slice_backward)
        self.AddObserver('KeyPressEvent', self.key_press_event)
        self.image_viewer = image_viewer
        self.status_actor = status_actor
        self.slice = image_viewer.GetSliceMin()
        self.min_slice = image_viewer.GetSliceMin()
        self.max_slice = image_viewer.GetSliceMax()
        self.update_status_message()

    def update_status_message(self):
        # Update the status message with the current slice
        message = f'Slice Number {self.slice + 1}/{self.max_slice + 1}'
        self.status_actor.GetMapper().SetInput(message)

    def move_slice_forward(self, obj, event):
        if self.slice < self.max_slice:
            self.slice += 1
            self.image_viewer.SetSlice(self.slice)
            self.update_status_message()
            self.image_viewer.Render()

    def move_slice_backward(self, obj, event):
        if self.slice > self.min_slice:
            self.slice -= 1
            self.image_viewer.SetSlice(self.slice)
            self.update_status_message()
            self.image_viewer.Render()

    def key_press_event(self, obj, event):
        key = self.GetInteractor().GetKeySym()
        if key == 'Up':
            self.move_slice_forward(obj, event)
        elif key == 'Down':
            self.move_slice_backward(obj, event)


class NiftiSliceViewer:
    def __init__(self, nifti_file, view_orientation='axial'):
        self.nifti_file = nifti_file
        self.view_orientation = view_orientation  # New attribute for view orientation
        self.reader = None
        self.shifted_data = None
        self.colors = vtk.vtkNamedColors()
        self.setup_reader()
        self.setup_data_shift()
        self.setup_reslice()  # Call setup_reslice instead of setup_viewer directly
        self.setup_text_labels()
        self.configure_interactor()

    def setup_reader(self):
        self.reader = vtk.vtkNIFTIImageReader()
        self.reader.SetFileName(self.nifti_file)
        self.reader.Update()

    def setup_data_shift(self):
        self.shifted_data = vtk.vtkImageShiftScale()
        self.shifted_data.SetInputConnection(self.reader.GetOutputPort())
        self.shifted_data.SetShift(-1000)
        self.shifted_data.SetScale(1.0)
        self.shifted_data.Update()

    def setup_reslice(self):
        self.reslice = vtk.vtkImageReslice()
        self.reslice.SetInputConnection(self.shifted_data.GetOutputPort())

        # Set the output spacing to be 1, 1, 1. This is not strictly necessary but can be useful.
        self.reslice.SetOutputSpacing(1, 1, 1)

        if self.view_orientation == 'coronal':
            self.reslice.SetResliceAxesDirectionCosines(1,0,0, 0,0,1, 0,-1,0)
        elif self.view_orientation == 'sagittal':
            self.reslice.SetResliceAxesDirectionCosines(0,1,0, 0,0,1, 1,0,0)
        # Default is axial; no changes needed

        self.setup_viewer()

    def setup_viewer(self):
        self.image_viewer = vtk.vtkImageViewer2()
        self.image_viewer.SetInputConnection(self.reslice.GetOutputPort())
        self.image_viewer.SetColorWindow(500)   # like CT WW
        self.image_viewer.SetColorLevel(50)
        # Rest of the setup_viewer method remains unchanged


    def setup_text_labels(self):
        self.slice_text_actor = self.create_text_actor("", 15, 10, 20, align_bottom=True)
        self.usage_text_actor = self.create_text_actor(
            "- Slice with mouse wheel or Up/Down-Key\n- Zoom with pressed right mouse button while dragging",
            0.05, 0.95, 14, normalized=True)

    def configure_interactor(self):
        self.interactor = vtk.vtkRenderWindowInteractor()
        self.interactor_style = CustomInteractorStyle(self.image_viewer, self.slice_text_actor)
        self.image_viewer.SetupInteractor(self.interactor)
        self.interactor.SetInteractorStyle(self.interactor_style)

    def create_text_actor(self, text, x, y, font_size, align_bottom=False, normalized=False):
        text_prop = vtk.vtkTextProperty()
        text_prop.SetFontFamilyToCourier()
        text_prop.SetFontSize(font_size)
        text_prop.SetVerticalJustificationToBottom() if align_bottom else text_prop.SetVerticalJustificationToTop()
        text_prop.SetJustificationToLeft()

        text_mapper = vtk.vtkTextMapper()
        text_mapper.SetInput(text)
        text_mapper.SetTextProperty(text_prop)

        text_actor = vtk.vtkActor2D()
        text_actor.SetMapper(text_mapper)
        if normalized:
            text_actor.GetPositionCoordinate().SetCoordinateSystemToNormalizedDisplay()
        text_actor.SetPosition(x, y)

        return text_actor

    def update_status_message(self, message):
        self.slice_text_actor.GetMapper().SetInput(message)

    def render(self):
        self.image_viewer.GetRenderer().AddActor2D(self.slice_text_actor)
        self.image_viewer.GetRenderer().AddActor2D(self.usage_text_actor)
        self.image_viewer.Render()
        self.image_viewer.GetRenderer().ResetCamera()
        self.image_viewer.GetRenderer().SetBackground(self.colors.GetColor3d('Black'))
        self.image_viewer.GetRenderWindow().SetSize(800, 800)
        self.image_viewer.GetRenderWindow().SetWindowName('NIFTI Viewer')
        self.interactor.Start()


if __name__ == '__main__':
    nifti_file = 'G:\\My Drive\\data\\Mosamatic\\testdata\\CT_NIFTI\\patient1_Abdomen_PVP_3.0_Br40_2_20200124092125_2.nii'
    viewer = NiftiSliceViewer(nifti_file=nifti_file, view_orientation='axial')
    viewer.render()


import os
import vtk
import pydicom
from PySide6.QtWidgets import QWidget, QVBoxLayout, QMessageBox
from mosamatic2.ui.widgets.panels.visualizations.slicevisualization.custominteractorstyle import CustomInteractorStyle
from vtkmodules.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor

COLOR_WINDOW = 400
COLOR_LEVEL = 40
IMAGE_SHIFT_SCALE = -1000


class SliceViewer(QWidget):
    def __init__(self, color_window=COLOR_WINDOW, color_level=COLOR_LEVEL):
        super(SliceViewer, self).__init__()
        self._nifti_file_or_dicom_dir = None
        self._view_orientation = 'axial'
        self._color_window = color_window
        self._color_level = color_level
        self._vtk_widget = QVTKRenderWindowInteractor(self)
        self._render_window = self._vtk_widget.GetRenderWindow()
        self._interactor = self._render_window.GetInteractor()
        self._interactor_style = None
        layout = QVBoxLayout()
        layout.addWidget(self._vtk_widget)
        self.setLayout(layout)
        self._default_renderer = vtk.vtkRenderer()
        self._default_renderer.SetBackground(0.0, 0.0, 0.0)  # black
        self._render_window.AddRenderer(self._default_renderer)
        self._render_window.Render()

    def color_window(self):
        return self._color_window

    def set_color_window(self, color_window):
        self._color_window = color_window
        if self._interactor_style:
            self._interactor_style.set_color_window(self._color_window)

    def color_level(self):
        return self._color_level

    def set_color_level(self, color_level):
        self._color_level = color_level
        if self._interactor_style:
            self._interactor_style.set_color_level(self._color_level)

    def nifti_file_or_dicom_dir(self):
        return self._nifti_file_or_dicom_dir
    
    def set_nifti_file_or_dicom_dir(self, nifti_file_or_dicom_dir):
        self._nifti_file_or_dicom_dir = nifti_file_or_dicom_dir

    def create_text_actor(self, text, x, y, font_size, align_bottom=False, normalized=False):
        text_prop = vtk.vtkTextProperty()
        text_prop.SetFontFamilyToCourier()
        text_prop.SetFontSize(font_size)
        text_prop.SetVerticalJustificationToBottom() if align_bottom else text_prop.SetVerticalJustificationToTop()
        text_prop.SetJustificationToLeft()
        text_mapper = vtk.vtkTextMapper()
        text_mapper.SetInput(text)
        text_mapper.SetTextProperty(text_prop)
        text_actor = vtk.vtkActor2D()
        text_actor.SetMapper(text_mapper)
        if normalized:
            text_actor.GetPositionCoordinate().SetCoordinateSystemToNormalizedDisplay()
        text_actor.SetPosition(x, y)
        return text_actor
    
    def is_nifti_file(self, file_path):
        return file_path.endswith('.nii') or file_path.endswith('.nii.gz')
    
    def is_dicom_dir(self, dir_path):
        first_dicom_file = os.path.join(dir_path, os.listdir(dir_path)[0])
        return pydicom.dcmread(first_dicom_file, stop_before_pixels=True)

    def load_image(self):
        if not self.nifti_file_or_dicom_dir():
            QMessageBox.warning(self, 'Warning', 'No NIFTI file or DICOM directory set')
            return
        if self.is_nifti_file(self.nifti_file_or_dicom_dir()):
            reader = vtk.vtkNIFTIImageReader()
            reader.SetFileName(self.nifti_file_or_dicom_dir())
        elif self.is_dicom_dir(self.nifti_file_or_dicom_dir()):
            reader = vtk.vtkDICOMImageReader()
            reader.SetDirectoryName(self.nifti_file_or_dicom_dir())
        reader.Update()
        image_data = reader.GetOutput()
        xmin, xmax, ymin, ymax, zmin, zmax = image_data.GetExtent()
        axial_index = (zmin + zmax) // 2
        slice_mapper = vtk.vtkImageSliceMapper()
        slice_mapper.SetInputData(image_data)
        slice_mapper.SetOrientationToZ()  # axial orientation
        slice_mapper.SetSliceNumber(axial_index)
        slice = vtk.vtkImageSlice()
        slice.GetProperty().SetColorWindow(self.color_window())
        slice.GetProperty().SetColorLevel(self.color_level())
        slice.SetMapper(slice_mapper)
        slice_text_actor = self.create_text_actor("", 0.01, 0.01, 12, align_bottom=True, normalized=True)
        usage_text_actor = self.create_text_actor(
            "- Slice with mouse wheel or Up/Down keys (first click inside viewer)\n"
            "- Zoom with pressed right mouse button while dragging\n"
            "- Pan with Shift key and left mouse button while dragging\n"
            "- Change contrast/brightness with pressed left mouse while dragging",
            0.01, 0.99, 12, normalized=True)
        ren = vtk.vtkRenderer()
        ren.AddActor2D(slice_text_actor)
        ren.AddActor2D(usage_text_actor)
        ren.AddViewProp(slice)
        ren.ResetCamera()
        self._render_window.RemoveRenderer(self._default_renderer)
        self._render_window.AddRenderer(ren)
        self._interactor_style = CustomInteractorStyle(image_data, slice_mapper, slice_text_actor, slice)
        self._interactor.SetInteractorStyle(self._interactor_style)
        self._interactor.Initialize()
        self._render_window.Render()


import os
import json
import zipfile
import shutil
import pydicom
from webdav3.client import Client


PROJECT_ID = 'P000000420'    # PORSCH trial
COLLECTION_ID = 'C000000002' # PORSCH trial CT scans
L3_BASE_DIR = 'L:\\FHML_SURGERY\\Mosamatic\\Projects\\P0009_PORSCH (Nicole Hildebrand)\\L3'
DOWNLOAD_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\PORSCH\\ZIPS'
SCANS_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\PORSCH\\SCANS'


def get_webdav_token():
    token_file_path = 'C:\\Users\\r.brecheisen\\mdr-webdav.txt'
    with open(token_file_path, 'r') as f:
        token = f.readline().strip()
    return token


def get_folder_name_lookup():
    folder_names_file_path = 'folder_names.json'
    with open(folder_names_file_path, 'r') as f:
        folder_name_lookup = json.load(f)
    return folder_name_lookup


def get_folder_patient_nrs():
    folder_name_lookup = get_folder_name_lookup()
    patient_nrs = {}
    for local_folder, value in folder_name_lookup.items():
        patient_nrs[local_folder] = []
        local_folder_path = os.path.join(L3_BASE_DIR, local_folder)
        for f in os.listdir(local_folder_path):
            try:
                patient_nr = f.split('.')[0]
                int(patient_nr) # Only use L3 files with whole numbers as patient ID
                patient_nrs[local_folder].append(patient_nr)
            except:
                pass
    return patient_nrs


def get_folder_l3s():
    folder_name_lookup = get_folder_name_lookup()
    l3s = {}
    for local_folder, value in folder_name_lookup.items():
        l3s[local_folder] = {}
        local_folder_path = os.path.join(L3_BASE_DIR, local_folder)
        for f in os.listdir(local_folder_path):
            try:
                patient_nr = f.split('.')[0]
                int(patient_nr)
                f_path = os.path.join(local_folder_path, f)
                l3s[local_folder][patient_nr] = f_path
            except:
                pass
    return l3s


def get_series_instance_uid(f):
    p = pydicom.dcmread(f, stop_before_pixels=True)
    return p.SeriesInstanceUID


def get_client():
    options = {
        'webdav_hostname': "https://download.datahubmaastricht.nl",
        'webdav_login':    "rbrecheise",
        'webdav_password': get_webdav_token(),
    }
    client = Client(options)
    return client


def get_l3s_and_scans(local_folder):
    folder_name_lookup = get_folder_name_lookup()
    folder_patient_nrs = get_folder_patient_nrs()
    folder_l3s = get_folder_l3s()
    client = get_client()
    l3s_and_scans = {}
    remote_folder = folder_name_lookup[local_folder]
    remote_folder_path = f'{PROJECT_ID}/{COLLECTION_ID}/{remote_folder}'
    patient_nrs = folder_patient_nrs[local_folder]
    # Run through patient numbers
    for patient_nr in patient_nrs:
        print(f'Processing patient {patient_nr}')
        zip_file = f'{patient_nr}.zip'
        zip_file_path = f'{remote_folder_path}/{zip_file}'
        # Check that ZIP file path exists on remote
        if client.check(zip_file_path):
            # Download ZIP file
            local_zip_dir = os.path.join(DOWNLOAD_DIR, local_folder)
            os.makedirs(local_zip_dir, exist_ok=True)
            local_zip_file_path = os.path.join(local_zip_dir, zip_file)
            client.download_sync(zip_file_path, local_zip_file_path)
            # Unpack ZIP file
            with zipfile.ZipFile(local_zip_file_path, 'r') as zip_ref:
                zip_ref.extractall(DOWNLOAD_DIR)
            l3_file = folder_l3s[local_folder][patient_nr]
            l3_series_instance_uid = get_series_instance_uid(l3_file)
            # Search for CT scan corresponding to L3 file
            stop_search = False
            for root, dirs, files in os.walk(os.path.join(local_zip_dir, patient_nr)):
                for f in files:
                    f_path = os.path.join(root, f)
                    try:
                        p = pydicom.dcmread(f_path, stop_before_pixels=True)
                        if l3_series_instance_uid == p.SeriesInstanceUID:
                            scan_dir = os.path.split(f_path)[0]
                            l3s_and_scans[l3_file] = scan_dir
                            stop_search = True
                            break
                    except:
                        pass
                if stop_search:
                    break
    return l3s_and_scans


folder_name_lookup = get_folder_name_lookup()
for local_folder, remote_folder in folder_name_lookup.items():

    # Check if JSON file exists. If so, load mapping from JSON
    json_file = local_folder.replace(' ', '_').lower() + '.json'
    if os.path.isfile(json_file):
        with open(json_file, 'r') as f:
            l3s_and_scans = json.load(f)
    else:
        # # Get mapping from L3 files to scan directories
        # l3s_and_scans = get_l3s_and_scans(local_folder)
        # # Save mapping to JSON
        # with open(json_file, 'w') as f:
        #     json.dump(l3s_and_scans, f)
        pass

    # Copy scan directory files to new directory with L3 file name
    for l3_file, scan_dir in l3s_and_scans.items():
        patient_nr = os.path.split(l3_file)[1].split('.')[0]
        print(f'Copying {scan_dir}')
        os.makedirs(os.path.join(SCANS_DIR, patient_nr), exist_ok=True)
        for f in os.listdir(scan_dir):
            f_path = os.path.join(scan_dir, f)
            shutil.copy(f_path, os.path.join(SCANS_DIR, patient_nr))


from PySide6.QtWidgets import (
    QTextEdit,
)
from mosamatic2.ui.widgets.panels.defaultpanel import DefaultPanel
from mosamatic2.core.managers.logmanager import LogManager

LOG = LogManager()

PANEL_TITLE = 'SettingsPanel'
PANEL_NAME = 'settingspanel'


class SettingsPanel(DefaultPanel):
    def __init__(self):
        super(SettingsPanel, self).__init__()
        self.set_title(PANEL_TITLE)


import os
import shutil
import zipfile
import utils
import constants
import dicom2nifti

DONE = [
    "Amphia Ziekenhuis (18)",
    "AUMC", # (11)
    "Catharina Ziekenhuis (24)",
    "Erasmus MC (17)",
    "LUMC (28)",
    "Isala Ziekenhuis (21)",
    "Jeroen Bosch Ziekenhuis (27)",
    "Medisch Spectrum Twente (22)",
    "MUMC+ (26)", # ERROR (could not find CT scan corresponding to L3)
    "OLVG Amsterdam (15)",
    "Radboud UMC (23)",
    "St. Antonius (13)", # ERROR (could not find CT scan corresponding to L3)
    "Tjongerschans Heerenveen (19)",
    "UMC Utrecht (14)",
    "UMCG (20)",
]

HOSPITAL_NAMES = [
]


def download_zip_file(client, remote_zip_file_path, local_zip_file_path):
    print(f'Downloading {remote_zip_file_path}...')
    client.download_sync(remote_path=remote_zip_file_path, local_path=local_zip_file_path)
    return local_zip_file_path


def unpack_zip_file(local_zip_file_path, local_unzipped_dir, patient_id):
    print(f'Unpacking {local_zip_file_path} to {local_unzipped_dir}...')
    with zipfile.ZipFile(local_zip_file_path, 'r') as zip_ref:
        zip_ref.extractall(local_unzipped_dir)
    local_unzipped_dir_path = os.path.join(local_unzipped_dir, patient_id)
    return local_unzipped_dir_path


def get_series_instance_uid(file_path):
    series_instance_uid = utils.get_series_instance_uid(file_path)
    return series_instance_uid


def find_scan_with_series_instance_uid(local_unzipped_dir_path, l3_series_instance_uid, patient_id):
    print(f'Finding CT scan in {local_unzipped_dir_path} with SUID = {l3_series_instance_uid}...')
    local_scan_dir_path = None
    found = False
    for root, dirs, files in os.walk(local_unzipped_dir_path):
        for f in files:
            f_path = os.path.join(root, f)
            if utils.is_valid_dicom(f_path):
                series_instance_uid = utils.get_series_instance_uid(f_path)
                if series_instance_uid == l3_series_instance_uid:
                    parent_dir_path = utils.get_parent_path(f_path)
                    local_scan_dir_path = os.path.join(constants.LOCAL_SCAN_DIR, patient_id)
                    shutil.copytree(parent_dir_path, local_scan_dir_path, dirs_exist_ok=True)
                    print(f'Found CT scan')
                    found = True
                    break
        if found:
            break
    return local_scan_dir_path


def convert_scan_to_nifti(local_scan_dir_path, local_scan_nifti_dir, patient_id):
    print(f'Converting CT scan to NIFTI...')
    local_nifti_file_path = os.path.join(local_scan_nifti_dir, f'{patient_id}.nii.gz')
    dicom2nifti.dicom_series_to_nifti(local_scan_dir_path, local_nifti_file_path)
    return local_nifti_file_path


def delete_temp_dirs(local_zip_file_path, local_unzipped_dir_path, local_scan_dir_path):
    print(f'Deleting temporary directories...')
    try:
        os.remove(local_zip_file_path)
        if local_unzipped_dir_path:
            shutil.rmtree(local_unzipped_dir_path)
        if local_scan_dir_path:
            shutil.rmtree(local_scan_dir_path)
    except Exception as e:
        print(f'Error cleaning up')


def process_file(client, l3_file_path, remote_zip_file_path):
    patient_id = os.path.split(l3_file_path)[1][:-4]
    remote_zip_file_name = os.path.split(remote_zip_file_path)[1]
    local_zip_file_path = os.path.join(constants.LOCAL_ZIP_DIR, remote_zip_file_name)
    local_unzipped_dir_path = None
    local_scan_dir_path = None

    try:
        local_zip_file_path = download_zip_file(client, remote_zip_file_path, local_zip_file_path)
        local_unzipped_dir_path = unpack_zip_file(local_zip_file_path, constants.LOCAL_UNZIPPED_DIR, patient_id)
        l3_series_instance_uid = get_series_instance_uid(l3_file_path)
        if l3_series_instance_uid:
            local_scan_dir_path = find_scan_with_series_instance_uid(local_unzipped_dir_path, l3_series_instance_uid, patient_id)
            if local_scan_dir_path:
                convert_scan_to_nifti(local_scan_dir_path, constants.LOCAL_SCAN_NIFTI_DIR, patient_id)
            else:
                print(f'Could not find CT scan in unpacked ZIP file directory')
        else:
            print(f'Could not find SeriesInstanceUID in L3 file path: {l3_file_path}')
    except Exception as e:
        print(f'Error occurred ({e}). Skipping...')
    finally:
        delete_temp_dirs(local_zip_file_path, local_unzipped_dir_path, local_scan_dir_path)


def count_nr_scans_to_process(file_mapping):
    count = 0
    for l3_file_path, _ in file_mapping.items():
        hospital_name = utils.get_parent_name(l3_file_path)
        if hospital_name in HOSPITAL_NAMES:
            count += 1
    return count


def main():
    client = utils.get_webdav_client()
    file_mapping = utils.load_dictionary('filemapping.json')
    start_time = utils.current_time_in_seconds()

    count = count_nr_scans_to_process(file_mapping)
    print(f'Found {count} scans to process')

    for l3_file_path, remote_zip_file_path in file_mapping.items():
        local_hospital_name = utils.get_parent_name(l3_file_path)

        if local_hospital_name in HOSPITAL_NAMES:
            start_time_inner = utils.current_time_in_seconds()
            print(f'Processing {l3_file_path}...')
            process_file(client, l3_file_path, remote_zip_file_path)
            elapsed_time_inner = utils.elapsed_time_in_seconds(start_time_inner)
            print(f'Elapsed time: {utils.duration(elapsed_time_inner)}')

    elapsed_time = utils.elapsed_time_in_seconds(start_time)
    print(f'Total elapsed time: {utils.duration(elapsed_time)}')


if __name__ == '__main__':
    main()


import os
import shutil
import zipfile


DATA_DIR = 'E:\\'
OUTPUT_DIR = 'D:\\Mosamatic\\Lisa'
RENAMED_DIR = 'D:\\Mosamatic\\Lisa\\Renamed'
os.makedirs(RENAMED_DIR, exist_ok=True)


for f in os.listdir(DATA_DIR):
    if f.startswith('L3_MUMC') and f.endswith('.zip'):
        f_path = os.path.join(DATA_DIR, f)
        with zipfile.ZipFile(f_path) as zip_ref:
            output_dir = os.path.join(OUTPUT_DIR, f)
            zip_ref.extractall(output_dir)
            dicom_file = os.listdir(output_dir)[0]
            dicom_file_path = os.path.join(output_dir, dicom_file)
            new_dicom_file = f + '.dcm'
            new_dicom_file_path = os.path.join(RENAMED_DIR, new_dicom_file)
            shutil.copyfile(dicom_file_path, new_dicom_file_path)
            print(new_dicom_file_path)
            print(new_dicom_file_path)


import click

from mosamatic2.core.tasks import Dicom2NiftiTask


@click.command(help='Converts root directory with DICOM series to NIFTI format')
@click.option(
    '--scans', 
    required=True, 
    type=click.Path(exists=True), 
    help='Root directory with DICOM scans (one for each patient)',
)
@click.option(
    '--compressed', 
    type=click.BOOL, 
    default=True, 
    help='Compress with gzip [true|false] (default: True)'
)
@click.option(
    '--output', 
    required=True, 
    type=click.Path(), 
    help='Output directory'
)
@click.option(
    '--overwrite', 
    type=click.BOOL, 
    default=False, 
    help='Overwrite [true|false]'
)
def dicom2nifti(scans, compressed, output, overwrite):
    """
    Converts DICOM scans to NIFTI format.
    
    Parameters
    ----------
    --scans : str
        Root directory with DICOM scans (one subdirectory for each patient)

    --compressed : bool
        Whether to compress the NIFTI file with gzip or not (default: True)

    --output : str
        Path to output directory

    --overwrite : bool
        Overwrite contents output directory [true|false]
    """
    task = Dicom2NiftiTask(
        inputs={'scans': scans},
        params={'compressed': compressed},
        output=output,
        overwrite=overwrite,
    )
    task.run()


import os
from PySide6.QtWidgets import (
    QLineEdit,
    QHBoxLayout,
    QVBoxLayout,
    QFormLayout,
    QPushButton,
    QFileDialog,
)
from mosamatic2.ui.widgets.panels.visualizations.visualization import Visualization
from mosamatic2.ui.widgets.panels.visualizations.liversegmentvisualization.liversegmentviewer import LiverSegmentViewer
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.ui.settings import Settings
from mosamatic2.ui.utils import is_macos

LOG = LogManager()
PANEL_TITLE = 'LiverSegmentVisualization'
PANEL_NAME = 'liversegmentvisualization'


class LiverSegmentVisualization(Visualization):
    def __init__(self):
        super(LiverSegmentVisualization, self).__init__()
        self.set_title(PANEL_TITLE)
        self._liver_segment_actors = {}
        self._liver_volumes = {}
        self._liver_segments_dir_line_edit = None
        self._liver_segments_dir_select_button = None
        self._liver_volumes_file_line_edit = None
        self._liver_volumes_file_select_button = None
        self._patient_id_line_edit = None
        self._load_liver_data_button = None
        self._liver_segment_viewer = None
        self._form_layout = None
        self._settings = None
        self.init_layout()

    def liver_segments_dir_line_edit(self):
        if not self._liver_segments_dir_line_edit:
            self._liver_segments_dir_line_edit = QLineEdit(self.settings().get(f'{PANEL_NAME}/liver_segments_dir'))
        return self._liver_segments_dir_line_edit
    
    def liver_segments_dir_select_button(self):
        if not self._liver_segments_dir_select_button:
            self._liver_segments_dir_select_button = QPushButton('Select directory')
            self._liver_segments_dir_select_button.clicked.connect(self.handle_liver_segments_dir_select_button)
        return self._liver_segments_dir_select_button
    
    def liver_volumes_file_line_edit(self):
        if not self._liver_volumes_file_line_edit:
            self._liver_volumes_file_line_edit = QLineEdit(self.settings().get(f'{PANEL_NAME}/liver_volumes_file'))
        return self._liver_volumes_file_line_edit
    
    def liver_volumes_file_select_button(self):
        if not self._liver_volumes_file_select_button:
            self._liver_volumes_file_select_button = QPushButton('Select file')
            self._liver_volumes_file_select_button.clicked.connect(self.handle_liver_volumes_file_select_button)
        return self._liver_volumes_file_select_button
    
    def patient_id_line_edit(self):
        if not self._patient_id_line_edit:
            self._patient_id_line_edit = QLineEdit(self.settings().get(f'{PANEL_NAME}/patient_id'))
        return self._patient_id_line_edit
    
    def load_liver_data_button(self):
        if not self._load_liver_data_button:
            self._load_liver_data_button = QPushButton('Load')
            self._load_liver_data_button.clicked.connect(self.handle_load_liver_data_button)
        return self._load_liver_data_button    
    
    def liver_segment_viewer(self):
        if not self._liver_segment_viewer:
            self._liver_segment_viewer = LiverSegmentViewer()
        return self._liver_segment_viewer
    
    def form_layout(self):
        if not self._form_layout:
            self._form_layout = QFormLayout()
            if is_macos():
                self._form_layout.setFieldGrowthPolicy(QFormLayout.ExpandingFieldsGrow)
        return self._form_layout

    def settings(self):
        if not self._settings:
            self._settings = Settings()
        return self._settings

    def init_layout(self):
        liver_segments_layout = QHBoxLayout()
        liver_segments_layout.addWidget(self.liver_segments_dir_line_edit())
        liver_segments_layout.addWidget(self.liver_segments_dir_select_button())
        liver_volumes_layout = QHBoxLayout()
        liver_volumes_layout.addWidget(self.liver_volumes_file_line_edit())
        liver_volumes_layout.addWidget(self.liver_volumes_file_select_button())
        self.form_layout().addRow('Liver segments directory', liver_segments_layout)
        self.form_layout().addRow('Liver volumes CSV file', liver_volumes_layout)
        self.form_layout().addRow('Patient ID', self.patient_id_line_edit())
        layout = QVBoxLayout()
        layout.addLayout(self.form_layout())
        layout.addWidget(self.load_liver_data_button())
        layout.addWidget(self.liver_segment_viewer())
        self.setLayout(layout)
        self.setObjectName(PANEL_NAME)

    def handle_liver_segments_dir_select_button(self):
        last_directory = self.settings().get('last_directory')
        dir_path = QFileDialog.getExistingDirectory(dir=last_directory)
        if dir_path:
            self.liver_segments_dir_line_edit().setText(dir_path)
            self.settings().set('last_directory', dir_path)

    def handle_liver_volumes_file_select_button(self):
        last_directory = self.settings().get('last_directory')
        file_path, _ = QFileDialog.getOpenFileName(dir=last_directory)
        if file_path:
            self.liver_volumes_file_line_edit().setText(file_path)
            self.settings().set('last_directory', os.path.dirname(file_path))

    def handle_load_liver_data_button(self):
        self.liver_segment_viewer().load_segments_and_volumes(
            liver_segments_dir=self.liver_segments_dir_line_edit().text(),
            liver_volumes_file=self.liver_volumes_file_line_edit().text(),
            patient_id=self.patient_id_line_edit().text(),
        )

    def save_inputs_and_parameters(self):
        self.settings().set(f'{PANEL_NAME}/liver_segments_dir', self.liver_segments_dir_line_edit().text())
        self.settings().set(f'{PANEL_NAME}/liver_volumes_file', self.liver_volumes_file_line_edit().text())


import pydicom
import pydicom.errors
import numpy as np
from pydicom.multival import MultiValue
from mosamatic2.core.managers.logmanager import LogManager

LOG = LogManager()


class DicomLoader:
    def __init__(self, file_path):
        self._file_path = file_path
        self._p = None
        self._slope = 1.0
        self._intercept = 0.0

    def load(self, stop_before_pixels=False):
        try:
            self._p = pydicom.dcmread(self._file_path, stop_before_pixels=stop_before_pixels)
            self._slope = float(getattr(self._p, "RescaleSlope", 1.0))
            self._intercept = float(getattr(self._p, "RescaleIntercept", 0.0))
            return True
        except pydicom.errors.InvalidDicomError as e:
            LOG.warning(f'File {self._file_path} is not a valid DICOM file ({e})')
            return False

    def load_as_numpy_and_u8disp(self):
        result = self.load()
        if not result:
            LOG.warning(f'Failed to load DICOM file')
            return None, None, None
        pixels = self._p.pixel_array
        pixels = pixels * self._slope + self._intercept

        # Window/level if present
        wc = getattr(self._p, "WindowCenter", None)
        ww = getattr(self._p, "WindowWidth", None)

        def _mv_first(x):
            if isinstance(x, MultiValue):
                return float(x[0])
            return float(x)

        if wc is not None and ww is not None:
            c = _mv_first(wc)
            w = _mv_first(ww)
            lo = c - (w / 2.0)
            hi = c + (w / 2.0)
        else:
            lo, hi = np.percentile(pixels, (1, 99))
            if hi <= lo:
                lo, hi = float(pixels.min()), float(pixels.max() + 1e-6)

        disp = np.clip((pixels - lo) / (hi - lo), 0.0, 1.0)
        disp8 = (disp * 255.0).astype(np.uint8)

        meta = {
            "SOPInstanceUID": getattr(self._p, "SOPInstanceUID", ""),
            "SeriesInstanceUID": getattr(self._p, "SeriesInstanceUID", ""),
            "StudyInstanceUID": getattr(self._p, "StudyInstanceUID", ""),
            "PixelSpacing": [float(x) for x in getattr(self._p, "PixelSpacing", [])] if hasattr(self._p, "PixelSpacing") else [],
            "Shape": list(disp8.shape),
        }
        return pixels, disp8, meta


import os
import numpy as np
import nibabel as nb
import pandas as pd
import matplotlib.pyplot as plt
from mosamatic2.core.tasks.task import Task
from mosamatic2.core.managers.logmanager import LogManager

LOG = LogManager()


class CalculateMaskStatisticsTask(Task):
    INPUTS = ['scans', 'masks']
    PARAMS = []

    def __init__(self, inputs, params, output, overwrite):
        super(CalculateMaskStatisticsTask, self).__init__(inputs, params, output, overwrite)

    def load_scans(self):
        scans = []
        for f in os.listdir(self.input('scans')):
            if f.endswith('.nii.gz'):
                scan = os.path.join(self.input('scans'), f)
                if os.path.isfile(scan):
                    scans.append(scan)
        return scans

    def load_mask_files(self):
        mask_files = []
        for f in os.listdir(self.input('masks')):
            if f.endswith('.nii.gz') and 'liver_segment' in f:
                mask_file = os.path.join(self.input('masks'), f)
                if os.path.isfile(mask_file):
                    mask_files.append(mask_file)
        return mask_files
    
    def collect_scan_mask_file_pairs(self, scans, mask_files):
        scan_mask_file_pairs = []
        for scan in scans:
            scan_name = os.path.split(scan)[1][:-7]
            for mask_file in mask_files:
                mask_file_name = os.path.split(mask_file)[1][:-7]
                if scan_name in mask_file_name:
                    scan_mask_file_pairs.append((scan, mask_file))
        return scan_mask_file_pairs
    
    def get_masked_voxels(self, scan_mask_file_pair):
        scan_image_file = scan_mask_file_pair[0]
        scan_image = nb.load(scan_image_file)
        scan_image_data = scan_image.get_fdata()
        mask_file_image_file = scan_mask_file_pair[1]
        mask_file_image_file_name = os.path.split(mask_file_image_file)[1]
        mask_file_image = nb.load(mask_file_image_file)
        mask_file_image_data = mask_file_image.get_fdata()
        mask = mask_file_image_data > 0
        masked_voxels = scan_image_data[mask]
        return masked_voxels, mask_file_image_file_name
    
    def calculate_volume_in_mL(self, mask_file):
        nifti_image = nb.load(mask_file)
        nifti_image_data = nifti_image.get_fdata()
        voxel_dims = nifti_image.header.get_zooms()[:3]
        voxel_volume = np.prod(voxel_dims)
        num_voxels = np.sum(nifti_image_data > 0.5)
        mask_volume = num_voxels * voxel_volume / 1000.0
        return mask_volume
    
    def create_histogram_png(self, masked_voxels, file_name):
        histogram_png_file = os.path.join(self.output(), f'{file_name}.png')
        plt.figure(figsize=(8,6))
        plt.hist(masked_voxels, bins=100, color='steelblue', edgecolor='black')
        plt.title(f'Histogram of HU values inside {file_name}')
        plt.xlabel('HU')
        plt.ylabel('Frequency')
        plt.savefig(histogram_png_file, dpi=300, bbox_inches='tight')
        plt.close()

    def save_data_to_file(self, data):
        csv_file_path = os.path.join(self.output(), 'statistics.csv')
        xls_file_path = os.path.join(self.output(), 'statistics.xlsx')
        df = pd.DataFrame(data=data)
        df.to_csv(csv_file_path, index=False, sep=';')
        df.to_excel(xls_file_path, index=False, engine='openpyxl')

    def run(self):
        scans = self.load_scans()
        mask_files = self.load_mask_files()
        scan_mask_file_pairs = self.collect_scan_mask_file_pairs(scans, mask_files)
        nr_steps = len(scan_mask_file_pairs)
        data = {
            'file': [],
            'mean_HU': [],
            'std_HU': [],
            'volume_mL': [],
        }
        for step in range(nr_steps):
            masked_voxels, file_name = self.get_masked_voxels(scan_mask_file_pairs[step])
            data['file'].append(file_name)
            data['mean_HU'].append(round(np.mean(masked_voxels)))
            data['std_HU'].append(round(np.std(masked_voxels)))
            data['volume_mL'].append(round(self.calculate_volume_in_mL(scan_mask_file_pairs[step][1])))
            self.create_histogram_png(masked_voxels, file_name)
            self.set_progress(step, nr_steps)
        self.save_data_to_file(data)


import numpy as np
from PySide6.QtWidgets import (
    QGraphicsPixmapItem,
    QGraphicsScene,
    QGraphicsView,
    QGraphicsEllipseItem,
    QRubberBand,
    QToolTip,
)
from PySide6.QtCore import Qt, QRectF, QTimer, Signal, QPoint, QRect
from PySide6.QtGui import (
    QPixmap,
    QPainter,
    QPen,
    QColor,
)
from mosamatic2.ui.widgets.panels.tools.segmentationeditor.utils import (
    clamp,
    circle_brush,
    np_u8_gray_to_qimage,
    np_rgba_to_qimage,
)

LABEL_COLORS = {
    0: (0, 0, 0),      # background
    1: (255, 0, 0),    # muscle
    5: (255, 255, 0),  # vat
    7: (0, 255, 255),  # sat
}


class SegmentationEditorGraphicsView(QGraphicsView):

    # SIGNALS

    active_label_changed = Signal(int)
    brush_radius_changed = Signal(int)
    smart_paint_changed = Signal(bool)
    fix_non_active_labels_changed = Signal(bool)

    #-------------------------------------------------------------------------------------------------------
    def __init__(self, 
                 active_label, 
                 brush_radius, 
                 smart_paint, 
                 fix_non_active_labels,
                 overall_opacity,
                 hu_lo_muscle,
                 hu_hi_muscle,
                 hu_lo_vat,
                 hu_hi_vat,
                 hu_lo_sat,
                 hu_hi_sat,
                 parent,
        ):
        super(SegmentationEditorGraphicsView, self).__init__(parent)

        # Fields
        self._document = None
        self._active_label = active_label
        self._brush_radius = brush_radius
        self._smart_paint = smart_paint
        self._fix_non_active_labels = fix_non_active_labels
        self._overall_opacity = overall_opacity
        self._hu_lo_muscle = hu_lo_muscle
        self._hu_hi_muscle = hu_hi_muscle
        self._hu_lo_vat = hu_lo_vat
        self._hu_hi_vat = hu_hi_vat
        self._hu_lo_sat = hu_lo_sat
        self._hu_hi_sat = hu_hi_sat

        # Undo
        self._undo_stack = []
        self._undo_limit = 50
        self._stroke_bbox = None
        self._stroke_before = None
        self._stroke_active = False

        # Graphics scene
        self._scene = None
        self._image_item = None
        self._image_overlay_item = None

        # Rendering state
        self._painting = False
        self._panning = False
        self._panning_anchor = None
        self._erase_mode = False
        self._space_down = False
        self._dirty = True
        self._update_timer = None
        self._zoom = 0
        self._brush_cursor_item = None

        # Zoom-rectangle (rubber band)
        self._zoom_rect_mode = False   # toggle on/off (e.g. with Z)
        self._zoom_rb = None
        self._zoom_origin = None       # QPoint (viewport coords)

        # Buffers must stay alive while QImages reference them
        self._disp8_ref = None
        self._disp_qimage_ref = None
        self._image_overlay_rgba_ref = None
        self._image_overlay_qimage_ref = None

        self.init()

    # INITIALIZATION

    #-------------------------------------------------------------------------------------------------------
    def init(self):
        self._scene = QGraphicsScene(self)
        self.setScene(self._scene)
        self.setBackgroundBrush(Qt.black)
        self._image_item = QGraphicsPixmapItem()
        self._image_overlay_item = QGraphicsPixmapItem()
        self._image_overlay_item.setZValue(10)
        self._scene.addItem(self._image_item)
        self._scene.addItem(self._image_overlay_item)
        self.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
        self.setFocus(Qt.FocusReason.OtherFocusReason)
        self.setRenderHints(self.renderHints() | QPainter.RenderHint.SmoothPixmapTransform)

        # Zoom rectangle
        self._zoom_rb = QRubberBand(QRubberBand.Shape.Rectangle, self.viewport())
        self._zoom_rb.hide()
        
        # Brush radius circle
        self.setMouseTracking(True)  # important: get mouseMoveEvent even w/o buttons
        self._brush_cursor_item = QGraphicsEllipseItem()
        self._brush_cursor_item.setZValue(1000)  # above overlay
        pen = QPen(QColor(255, 255, 0, 255))
        pen.setStyle(Qt.PenStyle.DashLine)
        pen.setWidth(1)
        pen.setCosmetic(True)  # keep constant on-screen width while zooming
        self._brush_cursor_item.setPen(pen)
        self._brush_cursor_item.setBrush(Qt.BrushStyle.NoBrush)
        self._brush_cursor_item.hide()
        self._scene.addItem(self._brush_cursor_item)
        
        # Cap refresh rate
        self._update_timer = QTimer(self)
        self._update_timer.setInterval(16)  # ~60fps cap
        self._update_timer.timeout.connect(self._maybe_update_overlay)
        self._update_timer.start()
        
        # Zooming
        self.setDragMode(QGraphicsView.DragMode.NoDrag)
        self.setTransformationAnchor(QGraphicsView.ViewportAnchor.AnchorUnderMouse)
        self.setResizeAnchor(QGraphicsView.ViewportAnchor.AnchorUnderMouse)

    # PUBLIC METHODS

    #-------------------------------------------------------------------------------------------------------
    def document(self):
        return self._document

    #-------------------------------------------------------------------------------------------------------
    def set_document(self, document):
        self._document = document
        self._set_base_image(self._document.disp8)
        self._rebuild_overlay_full()
        self.reset_zoom()

    #-------------------------------------------------------------------------------------------------------
    def set_active_label(self, active_label):
        self._active_label = active_label

    #-------------------------------------------------------------------------------------------------------
    def set_brush_radius(self, brush_radius):
        self._brush_radius = brush_radius
        vp_pos = self.mapFromGlobal(self.cursor().pos())
        self._update_brush_cursor_at_viewport_pos(vp_pos)

    #-------------------------------------------------------------------------------------------------------
    def set_smart_paint(self, smart_paint):
        self._smart_paint = smart_paint

    #-------------------------------------------------------------------------------------------------------
    def set_fix_non_active_labels(self, fix_non_active_labels):
        self._fix_non_active_labels = fix_non_active_labels

    #-------------------------------------------------------------------------------------------------------
    def set_hu_lo_muscle(self, hu):
        self._hu_lo_muscle = hu

    #-------------------------------------------------------------------------------------------------------
    def set_hu_hi_muscle(self, hu):
        self._hu_hi_muscle = hu

    #-------------------------------------------------------------------------------------------------------
    def set_hu_lo_vat(self, hu):
        self._hu_lo_vat = hu

    #-------------------------------------------------------------------------------------------------------
    def set_hu_hi_vat(self, hu):
        self._hu_hi_vat = hu

    #-------------------------------------------------------------------------------------------------------
    def set_hu_lo_sat(self, hu):
        self._hu_lo_sat = hu

    #-------------------------------------------------------------------------------------------------------
    def set_hu_hi_sat(self, hu):
        self._hu_hi_sat = hu

    #-------------------------------------------------------------------------------------------------------
    def set_overall_opacity(self, opacity):
        self._overall_opacity = opacity
        self._rebuild_overlay_full()

    #-------------------------------------------------------------------------------------------------------
    def clear_labels(self):
        if self._document is None:
            return
        self._document.mask.fill(0)
        self._rebuild_overlay_full()

    #-------------------------------------------------------------------------------------------------------
    def reset_zoom(self):
        if self._document is None:
            return
        h, w = self._document.disp8.shape
        self.setSceneRect(QRectF(0, 0, w, h))
        self.resetTransform()
        self._zoom = 0
        self.fitInView(QRectF(0, 0, w, h), Qt.AspectRatioMode.KeepAspectRatio)

    # PRIVATE HELPERS

    #-------------------------------------------------------------------------------------------------------
    def _zoom_to_viewport_rect(self, vp_rect: QRect):
        if self._document is None:
            return
        if vp_rect.width() < 5 or vp_rect.height() < 5:
            return

        # Convert viewport rect -> scene rect and zoom
        scene_rect = self.mapToScene(vp_rect).boundingRect()
        self.fitInView(scene_rect, Qt.AspectRatioMode.KeepAspectRatio)
        self._zoom += 1  # optional bookkeeping

    #-------------------------------------------------------------------------------------------------------
    def _update_brush_cursor_at_viewport_pos(self, vp_pos: QPoint):
        if self._document is None or self._brush_cursor_item is None:
            return

        scene_pos = self.mapToScene(vp_pos)
        item_pos = self._image_item.mapFromScene(scene_pos)

        x = item_pos.x()
        y = item_pos.y()

        h, w = self._document.mask.shape
        if not (0 <= x < w and 0 <= y < h):
            self._brush_cursor_item.hide()
            return

        r = self._brush_radius
        self._brush_cursor_item.setRect(QRectF(x - r, y - r, 2 * r, 2 * r))
        self._brush_cursor_item.show()    

    #-------------------------------------------------------------------------------------------------------
    def _union_bbox(self, a, b):
        # bboxes are (x0,y0,x1,y1) inclusive
        if a is None:
            return b
        ax0, ay0, ax1, ay1 = a
        bx0, by0, bx1, by1 = b
        return (min(ax0, bx0), min(ay0, by0), max(ax1, bx1), max(ay1, by1))

    #-------------------------------------------------------------------------------------------------------
    def _brush_bbox(self, x, y):
        h, w = self._document.mask.shape
        r = self._brush_radius
        x0 = clamp(x - r, 0, w - 1)
        x1 = clamp(x + r, 0, w - 1)
        y0 = clamp(y - r, 0, h - 1)
        y1 = clamp(y + r, 0, h - 1)
        return (x0, y0, x1, y1)

    #-------------------------------------------------------------------------------------------------------
    def _ensure_stroke_before_covers(self, new_bbox):
        """Ensure _stroke_before covers union(current_bbox, new_bbox)."""
        if self._stroke_bbox is None:
            self._stroke_bbox = new_bbox
            x0,y0,x1,y1 = new_bbox
            self._stroke_before = self._document.mask[y0:y1+1, x0:x1+1].copy()
            return

        union = self._union_bbox(self._stroke_bbox, new_bbox)
        if union == self._stroke_bbox:
            return

        # Expand before-snapshot to cover union by re-sampling from current mask,
        # then overwrite the already-captured area with the original "before" pixels.
        ux0,uy0,ux1,uy1 = union
        expanded = self._document.mask[uy0:uy1+1, ux0:ux1+1].copy()

        ox0,oy0,ox1,oy1 = self._stroke_bbox
        # placement of old before inside expanded
        ex0 = ox0 - ux0
        ey0 = oy0 - uy0
        expanded[ey0:ey0+(oy1-oy0+1), ex0:ex0+(ox1-ox0+1)] = self._stroke_before

        self._stroke_bbox = union
        self._stroke_before = expanded

    #-------------------------------------------------------------------------------------------------------
    def _label_hu_ranges(self):
        return {
            1: (self._hu_lo_muscle, self._hu_hi_muscle),
            5: (self._hu_lo_vat, self._hu_hi_vat),
            7: (self._hu_lo_sat, self._hu_hi_sat),
        }        

    #-------------------------------------------------------------------------------------------------------
    def _set_base_image(self, disp8):
        self._disp8_ref = np.ascontiguousarray(disp8)
        self._disp_qimage_ref = np_u8_gray_to_qimage(self._disp8_ref)
        pix = QPixmap.fromImage(self._disp_qimage_ref)
        self._image_item.setPixmap(pix)

    #-------------------------------------------------------------------------------------------------------
    def _rebuild_overlay_full(self):
        if self._document is None:
            return
        h, w = self._document.mask.shape
        rgba = np.zeros((h, w, 4), dtype=np.uint8)
        for lab, (r, g, b) in LABEL_COLORS.items():
            if lab == 0:
                continue
            m = (self._document.mask == lab)
            rgba[m, 0] = r
            rgba[m, 1] = g
            rgba[m, 2] = b
            rgba[m, 3] = int(255 * self._overall_opacity)

        # Keep refs alive
        self._image_overlay_rgba_ref = np.ascontiguousarray(rgba)
        self._image_overlay_qimage_ref = np_rgba_to_qimage(self._image_overlay_rgba_ref)
        self._image_overlay_item.setPixmap(QPixmap.fromImage(self._image_overlay_qimage_ref))
        self._dirty = False

    #-------------------------------------------------------------------------------------------------------
    def _maybe_update_overlay(self):
        if self._dirty and self._document is not None:
            # v1: full rebuild (fast enough for many single-slice cases)
            self._rebuild_overlay_full()

    #-------------------------------------------------------------------------------------------------------
    def _mouse_to_pixel(self, event):
        if self._document is None:
            return None
        scene_pos = self.mapToScene(event.position().toPoint())
        item_pos = self._image_item.mapFromScene(scene_pos)
        x = int(item_pos.x())
        y = int(item_pos.y())
        h, w = self._document.mask.shape
        if 0 <= x < w and 0 <= y < h:
            return x, y
        return None

    #-------------------------------------------------------------------------------------------------------
    def _stamp(self, x, y, label):
        if self._document is None:
            return
        
        if self._stroke_active:
            self._ensure_stroke_before_covers(self._brush_bbox(x, y))

        h, w = self._document.mask.shape
        brush = circle_brush(self._brush_radius)
        r = self._brush_radius

        x0 = clamp(x - r, 0, w - 1)
        x1 = clamp(x + r, 0, w - 1)
        y0 = clamp(y - r, 0, h - 1)
        y1 = clamp(y + r, 0, h - 1)

        # Corresponding brush window
        bx0 = x0 - (x - r)
        by0 = y0 - (y - r)
        bx1 = bx0 + (x1 - x0) + 1
        by1 = by0 + (y1 - y0) + 1

        region = self._document.mask[y0 : y1 + 1, x0 : x1 + 1]
        region_img = self._document.img[y0 : y1 + 1, x0 : x1 + 1]
        b = brush[by0:by1, bx0:bx1]

        if self._fix_non_active_labels:
            writable = (region == 0) | (region == label)
        else:
            writable = np.ones_like(region, dtype=bool)

        if label == 0:
            region[b & (region == self._active_label)] = 0
            self._dirty = True
            return
        
        if not self._smart_paint:
            region[b & writable] = label
            self._dirty = True
            return

        # Smart paint: only paint pixels whose intensity is in the label's range
        lo, hi = self._label_hu_ranges().get(label, (-np.inf, np.inf))
        valid = (region_img >= lo) & (region_img <= hi)
        region[b & writable & valid] = label
        self._dirty = True

    #-------------------------------------------------------------------------------------------------------
    def undo(self):
        if self._document is None or not self._undo_stack:
            return
        item = self._undo_stack.pop()
        x0,y0,x1,y1 = item["bbox"]
        self._document.mask[y0:y1+1, x0:x1+1] = item["before"]
        self._dirty = True

    # EVENT HANDLERS

    #-------------------------------------------------------------------------------------------------------
    def wheelEvent(self, event):
        if event.modifiers() & Qt.KeyboardModifier.ControlModifier:
            # Ctrl+Wheel -> brush size
            delta = event.angleDelta().y()
            step = 1 if delta > 0 else -1
            self._brush_radius = clamp(self._brush_radius + step, 1, 100)
            self.brush_radius_changed.emit(self._brush_radius)
            vp_pos = self.mapFromGlobal(self.cursor().pos())
            self._update_brush_cursor_at_viewport_pos(vp_pos)
            event.accept()
            return

        # Wheel -> zoom
        delta = event.angleDelta().y()
        if delta == 0:
            return
        factor = 1.25 if delta > 0 else 0.8
        self.scale(factor, factor)
        self._zoom += 1 if delta > 0 else -1
        event.accept()

    #-------------------------------------------------------------------------------------------------------
    def mousePressEvent(self, event):
        if self._document is None:
            return super().mousePressEvent(event)

        # Pan: middle mouse OR Space+Left
        if event.button() == Qt.MouseButton.MiddleButton or (
            event.button() == Qt.MouseButton.LeftButton and self._space_down
        ):
            self._panning = True
            self._panning_anchor = event.position().toPoint()
            self.setCursor(Qt.CursorShape.ClosedHandCursor)
            event.accept()
            return

        # Zoom rectangle: Left click drag when in zoom-rect mode
        if self._zoom_rect_mode and event.button() == Qt.MouseButton.LeftButton and not self._space_down:
            self._painting = False  # ensure we don't paint
            self._zoom_origin = event.position().toPoint()
            self._zoom_rb.setGeometry(QRect(self._zoom_origin, self._zoom_origin))
            self._zoom_rb.show()
            event.accept()
            return
        
        if event.button() in (Qt.MouseButton.LeftButton, Qt.MouseButton.RightButton):
            pos = self._mouse_to_pixel(event)
            if pos is None:
                return
            x, y = pos

            self._stroke_active = True
            self._stroke_bbox = None
            self._stroke_before = None

            # Right button always erase; left button paints active label (unless erase_mode)
            if event.button() == Qt.MouseButton.RightButton or self._erase_mode:
                self._stamp(x, y, 0)
            else:
                self._stamp(x, y, self._active_label)

            self._painting = True
            event.accept()
            return
        super().mousePressEvent(event)

    #-------------------------------------------------------------------------------------------------------
    def mouseMoveEvent(self, event):
        if self._document is None:
            return super().mouseMoveEvent(event)
        
        self._update_brush_cursor_at_viewport_pos(event.position().toPoint())
        
        if self._zoom_rb.isVisible() and self._zoom_origin is not None:
            cur = event.position().toPoint()
            self._zoom_rb.setGeometry(QRect(self._zoom_origin, cur).normalized())
            event.accept()
            return
            
        if self._panning and self._panning_anchor is not None:
            delta = event.position().toPoint() - self._panning_anchor
            self._panning_anchor = event.position().toPoint()
            self.horizontalScrollBar().setValue(self.horizontalScrollBar().value() - delta.x())
            self.verticalScrollBar().setValue(self.verticalScrollBar().value() - delta.y())
            event.accept()
            return

        if self._painting:
            pos = self._mouse_to_pixel(event)
            if pos is None:
                return
            x, y = pos
            buttons = event.buttons()

            if (event.modifiers() & Qt.KeyboardModifier.ShiftModifier) and (buttons & Qt.MouseButton.LeftButton):
                self._stamp(x, y, 0)
            elif buttons & Qt.MouseButton.RightButton or self._erase_mode:
                self._stamp(x, y, 0)
            elif buttons & Qt.MouseButton.LeftButton:
                self._stamp(x, y, self._active_label)

            event.accept()
            return
        super().mouseMoveEvent(event)

    #-------------------------------------------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        if self._zoom_rb.isVisible() and event.button() == Qt.MouseButton.LeftButton:
            rect = self._zoom_rb.geometry()
            self._zoom_rb.hide()
            self._zoom_to_viewport_rect(rect)
            self._zoom_origin = None
            event.accept()
            return
            
        if self._panning and event.button() in (Qt.MouseButton.MiddleButton, Qt.MouseButton.LeftButton):
            self._panning = False
            self._panning_anchor = None
            self.setCursor(Qt.CursorShape.ArrowCursor)
            event.accept()
            return
        
        if event.button() in (Qt.MouseButton.LeftButton, Qt.MouseButton.RightButton):
            # Commit one undo item per stroke
            if self._stroke_bbox is not None and self._stroke_before is not None:
                x0, y0, x1, y1 = self._stroke_bbox
                after = self._document.mask[y0:y1+1, x0:x1+1].copy()
                if not np.array_equal(after, self._stroke_before):
                    self._undo_stack.append({"bbox": self._stroke_bbox, "before": self._stroke_before, "after": after})
                    if len(self._undo_stack) > self._undo_limit:
                        self._undo_stack.pop(0)

            # End stroke capture
            self._stroke_active = False
            self._stroke_bbox = None
            self._stroke_before = None
            self._painting = False
            event.accept()
            return
        super().mouseReleaseEvent(event)

    #-------------------------------------------------------------------------------------------------------
    def keyPressEvent(self, event):
        key = event.key()

        if (event.modifiers() & Qt.KeyboardModifier.ControlModifier) and key == Qt.Key.Key_Z:
            self.undo()
            event.accept()
            return
        
        if key == Qt.Key.Key_U:
            self.undo()
            event.accept()
            return
        
        if key == Qt.Key.Key_Z:
            self._zoom_rect_mode = True
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), "Ready to zoom", self)
            # nice UX: hide cursor circle while zooming
            if self._zoom_rect_mode:
                self._brush_cursor_item.hide()
            event.accept()
            return
            
        if key == Qt.Key.Key_R:
            self.reset_zoom()
            event.accept()
            return

        if key in (Qt.Key.Key_BracketLeft, Qt.Key.Key_BracketRight):
            step = -1 if key == Qt.Key.Key_BracketLeft else 1
            self._brush_radius = clamp(self._brush_radius + step, 1, 100)
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), "Updating brush radius", self)
            self.brush_radius_changed.emit(self._brush_radius)
            vp_pos = self.mapFromGlobal(self.cursor().pos())
            self._update_brush_cursor_at_viewport_pos(vp_pos)
            event.accept()
            return

        if key == Qt.Key.Key_E:
            self._erase_mode = True
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), "Ready to erase", self)
            event.accept()
            return

        if key == Qt.Key.Key_1:
            self._active_label = 1
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), "Selected muscle", self)
            self.active_label_changed.emit(self._active_label)
            event.accept()
            return
        
        if key == Qt.Key.Key_5:
            self._active_label = 5
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), "Selected visceral fat", self)
            self.active_label_changed.emit(self._active_label)
            event.accept()
            return
        
        if key == Qt.Key.Key_7:
            self._active_label = 7
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), "Selected subcutaneous fat", self)
            self.active_label_changed.emit(self._active_label)
            event.accept()
            return
        
        if key == Qt.Key.Key_Space:
            self._space_down = True
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), "Reading for panning", self)
            event.accept()
            return

        if key == Qt.Key.Key_S:
            self._smart_paint = not self._smart_paint
            enabled = 'enabled' if self._smart_paint else 'disabled'
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), f"SmartPaint {enabled}", self)
            self.smart_paint_changed.emit(self._smart_paint)
            event.accept()
            return

        if key == Qt.Key.Key_F:
            self._fix_non_active_labels = not self._fix_non_active_labels
            # self._smart_paint = not self._smart_paint
            enabled = 'enabled' if self._smart_paint else 'disabled'
            QToolTip.showText(self.mapToGlobal(QPoint(10, 10)), f"Fix non-active labels: {enabled}", self)
            self.fix_non_active_labels_changed.emit(self._fix_non_active_labels)
            event.accept()
            return

        super().keyPressEvent(event)

    #-------------------------------------------------------------------------------------------------------
    def keyReleaseEvent(self, event):
        key = event.key()

        if key == Qt.Key.Key_Z:
            self._zoom_rect_mode = False
            vp_pos = self.mapFromGlobal(self.cursor().pos())
            self._update_brush_cursor_at_viewport_pos(vp_pos)
            event.accept()
            return
            
        if key == Qt.Key.Key_E:
            self._erase_mode = False
            event.accept()
            return
        
        if key == Qt.Key.Key_Space:
            self._space_down = False
            event.accept()
            return
        
        super().keyReleaseEvent(event)


import sys

from PySide6.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QWidget
from vtkmodules.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor

import vtk


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)

        # Central widget and layout
        self.frame = QWidget()
        self.layout = QVBoxLayout()
        self.frame.setLayout(self.layout)
        self.setCentralWidget(self.frame)

        # Create VTK widget
        self.vtk_widget = QVTKRenderWindowInteractor(self.frame)
        self.layout.addWidget(self.vtk_widget)

        # VTK renderer and scene setup
        self.renderer = vtk.vtkRenderer()
        self.vtk_widget.GetRenderWindow().AddRenderer(self.renderer)

        # Add a simple 3D object
        cone = vtk.vtkConeSource()
        cone.SetHeight(3.0)
        cone.SetRadius(1.0)
        cone.SetResolution(30)

        cone_mapper = vtk.vtkPolyDataMapper()
        cone_mapper.SetInputConnection(cone.GetOutputPort())

        cone_actor = vtk.vtkActor()
        cone_actor.SetMapper(cone_mapper)

        self.renderer.AddActor(cone_actor)
        self.renderer.SetBackground(0.1, 0.2, 0.4)

        # Initialize interactor
        self.interactor = self.vtk_widget.GetRenderWindow().GetInteractor()
        self.interactor.Initialize()
        self.interactor.Start()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec())



import os
import shutil
import pydicom
import numpy as np
import nibabel as nib
import ants
from totalsegmentator.python_api import totalsegmentator


DIXON_DIR = "D:\\Mosamatic\\MaximeDewulf\\MRI"
CT_DIR = "D:\\Mosamatic\\MaximeDewulf\\CT"
OUTPUT_DIXON_DIR = "D:\\Mosamatic\\MaximeDewulf\\Dixon"
OUTPUT_DIXON_WATER_DIR = "D:\\Mosamatic\\MaximeDewulf\\DixonWater"
CT_SERIES_DESCRIPTION = 'Abd_PVP_ThorAbd  3.0  Br40  3'
CT_SERIES_NUMBER = '7'
CT_PVP_DIR = "D:\\Mosamatic\\MaximeDewulf\\CT_PVP"
TOTAL_SEGMENTATOR_OUTPUT_DIR = "D:\\Mosamatic\\MaximeDewulf\\TS_OUTPUT"
TOTAL_SEGMENTATOR_OUTPUT_DIXON_DIR = "D:\\Mosamatic\\MaximeDewulf\\TS_OUTPUT_DIXON"

os.makedirs(OUTPUT_DIXON_DIR, exist_ok=True)
os.makedirs(OUTPUT_DIXON_WATER_DIR, exist_ok=True)
os.makedirs(CT_PVP_DIR, exist_ok=True)
os.makedirs(TOTAL_SEGMENTATOR_OUTPUT_DIR, exist_ok=True)
os.makedirs(TOTAL_SEGMENTATOR_OUTPUT_DIXON_DIR, exist_ok=True)


# def build_affine(ds_list):
#     # ds_list = list of dicoms in correct slice order
#     first = ds_list[0]
#     # voxel spacing
#     px_spacing = [float(x) for x in first.PixelSpacing]  # [row, col]
#     slice_thickness = float(getattr(first, "SliceThickness", 1.0))
#     # orientation
#     orient = [float(x) for x in first.ImageOrientationPatient]  # [row_x, row_y, row_z, col_x, col_y, col_z]
#     row_cosine = np.array(orient[0:3])
#     col_cosine = np.array(orient[3:6])
#     slice_cosine = np.cross(row_cosine, col_cosine)
#     # origin (first slice)
#     origin = np.array([float(x) for x in first.ImagePositionPatient])
#     # direction matrix
#     direction = np.vstack((row_cosine, col_cosine, slice_cosine)).T
#     # voxel spacing
#     spacing = np.array([px_spacing[0], px_spacing[1], slice_thickness])
#     # affine = direction * spacing
#     affine = np.eye(4)
#     affine[:3, :3] = direction * spacing
#     affine[:3, 3] = origin
#     return affine


def build_affine(ds_list):
    first = ds_list[0]
    # Pixel spacing in-plane
    px_spacing = [float(x) for x in first.PixelSpacing]  # [row, col]
    # Orientation
    orient = [float(x) for x in first.ImageOrientationPatient]
    row_cosine = np.array(orient[0:3])
    col_cosine = np.array(orient[3:6])
    slice_cosine = np.cross(row_cosine, col_cosine)
    # Origin
    origin = np.array([float(x) for x in first.ImagePositionPatient])
    # True slice spacing from positions
    if len(ds_list) > 1:
        pos_first = np.array(ds_list[0].ImagePositionPatient, dtype=float)
        pos_last  = np.array(ds_list[-1].ImagePositionPatient, dtype=float)
        num_slices = len(ds_list)
        z_spacing = np.dot(pos_last - pos_first, slice_cosine) / (num_slices - 1)
    else:
        z_spacing = float(getattr(first, "SliceThickness", 1.0))
    spacing = np.array([px_spacing[0], px_spacing[1], abs(z_spacing)])
    # Build affine
    direction = np.vstack((row_cosine, col_cosine, slice_cosine)).T
    affine = np.eye(4)
    affine[:3, :3] = direction * spacing
    affine[:3, 3] = origin
    return affine


def load_dicom_series(files, modality, series_nr=None):
    image_files = []
    for f in files:
        p = pydicom.dcmread(f, stop_before_pixels=True)
        if p.Modality == modality:
            if series_nr:
                if series_nr == p.SeriesNumber:
                    image_files.append(f)
            else:
                image_files.append(f)
    sorted_files = sorted(
        image_files,
        key=lambda f: int(pydicom.dcmread(f, stop_before_pixels=True).InstanceNumber)
    )
    sorted_p_files = [pydicom.dcmread(f, stop_before_pixels=True) for f in sorted_files]
    affine = build_affine(sorted_p_files)
    volume = []
    for path in sorted_files:
        ds = pydicom.dcmread(path)
        arr = ds.pixel_array.astype(np.float32)
        if 'RescaleSlope' in ds and 'RescaleIntercept' in ds:
            arr = arr * float(ds.RescaleSlope) + float(ds.RescaleIntercept)
        volume.append(arr)    
    return np.stack(volume, axis=0), affine


def test_totalsegmentator_dixon():

    # Save Dixon MRI water image to NIFTI
    dixon_ip = []
    dixon_op = []
    for root, dirs, files in os.walk(DIXON_DIR):
        for f in files:
            f_path = os.path.join(root, f)
            p = pydicom.dcmread(f_path, stop_before_pixels=True)
            if p.Modality == 'MR' and 'ImageType' in p and len(p.ImageType) > 4:
                if p.ImageType[4] == 'IN_PHASE':
                    dixon_ip.append(f_path)
                elif p.ImageType[4] == 'OPP_PHASE':
                    dixon_op.append(f_path)
                else:
                    pass
    dixon_ip, dixon_affine = load_dicom_series(dixon_ip, 'MR')
    dixon_op, _ = load_dicom_series(dixon_op, 'MR')
    dixon_water = 0.5 * (dixon_ip + dixon_op)
    dixon_water_file = os.path.join(OUTPUT_DIXON_WATER_DIR, 'dixon_water.nii.gz')
    # nib.save(nib.Nifti1Image(dixon_water, np.eye(4)), dixon_water_file)
    nib.save(nib.Nifti1Image(dixon_water, dixon_affine), dixon_water_file)

    ct = []
    for f in os.listdir(CT_DIR):
        f_path = os.path.join(CT_DIR, f)
        ct.append(f_path)
    ct, ct_affine = load_dicom_series(ct, 'CT', series_nr=CT_SERIES_NUMBER)
    ct_file = os.path.join(OUTPUT_DIXON_WATER_DIR, 'ct.nii.gz')
    # nib.save(nib.Nifti1Image(ct, np.eye(4)), ct_file)
    nib.save(nib.Nifti1Image(ct, ct_affine), ct_file)

    # Get liver mask from CT
    totalsegmentator(input=CT_PVP_DIR, output=TOTAL_SEGMENTATOR_OUTPUT_DIR, task='liver_segments')
    liver_mask_files = []
    for f in os.listdir(TOTAL_SEGMENTATOR_OUTPUT_DIR):
        f_path = os.path.join(TOTAL_SEGMENTATOR_OUTPUT_DIR, f)
        liver_mask_files.append(f_path)
    ref_img = nib.load(liver_mask_files[0])
    merged_data = np.zeros(ref_img.shape, dtype=bool)
    for f in liver_mask_files:
        mask_img = nib.load(f)
        mask_data = mask_img.get_fdata().astype(bool)
        merged_data = np.logical_or(merged_data, mask_data)
    merged_data = merged_data.astype(np.uint8)
    ct_liver_mask = nib.Nifti1Image(merged_data, ref_img.affine, ref_img.header)
    nib.save(ct_liver_mask, os.path.join(OUTPUT_DIXON_WATER_DIR, 'ct_liver_mask.nii.gz'))
    
    # Register CT and Dixon water image using ANTsPy
    dixon_water = ants.image_read(dixon_water_file)
    ct = ants.image_read(ct_file)
    reg = ants.registration(fixed=ct, moving=dixon_water, type_of_transform='SyN')
    dixon_water_registered = reg['warpedmovout']
    ants.image_write(dixon_water_registered, os.path.join(OUTPUT_DIXON_WATER_DIR, 'dixon_water_registered.nii.gz'))
    dixon_water_registered_resampled = ants.resample_image_to_target(dixon_water_registered, ct, interp_type='linear')
    ants.image_write(dixon_water_registered_resampled, os.path.join(OUTPUT_DIXON_WATER_DIR, 'dixon_water_registered_resampled.nii.gz'))


import os
import json
import constants
from pathlib import Path
from webdav3.client import Client


def get_webdav_token():
    token_file_path = 'C:\\Users\\r.brecheisen\\mdr-webdav.txt'
    with open(token_file_path, 'r') as f:
        token = f.readline().strip()
    return token


def get_webdav_client():
    options = {
        'webdav_hostname': "https://download.datahubmaastricht.nl",
        'webdav_login':    "rbrecheise",
        'webdav_password': get_webdav_token(),
    }
    client = Client(options)
    return client


def get_path(file_name):
    return os.path.join(constants.BASE_DIR, file_name)


def get_parent_path(file_path):
    parent = Path(file_path).parent
    return str(parent)


def get_parent_name(file_path):
    return Path(file_path).parent.name


def print_dictionary(dictionary):
    for k, v in dictionary.items():
        print(f'{k}: {v}')


def load_dictionary(file_name, reverse=False):
    dictionary = None
    with open(get_path(file_name), 'r') as f:
        dictionary = json.load(f)
    if not reverse:
        dictionary = {v: k for k, v in dictionary.items()}
    return dictionary


def save_dictionary(dictionary, file_name):
    with open(get_path(file_name), 'w') as f:
        json.dump(dictionary, f, indent=4)


def is_number(file_name):
    try:
        patient_nr = file_name.split('.')[0]
        int(patient_nr)
        return True
    except:
        return False


# %%
import torch
import torch.nn as nn
from torch.nn import functional as F
from torchsummary import summary


def double_conv(in_c, out_c, dropout_rate):
    conv = nn.Sequential(
        nn.Conv2d(in_c, out_c, kernel_size=3, padding="same"),
        nn.PReLU(),
        nn.BatchNorm2d(out_c),
        nn.Dropout(dropout_rate),
        nn.Conv2d(out_c, out_c, kernel_size=3, padding="same"),
        nn.PReLU(),
        nn.BatchNorm2d(out_c),
    )
    return conv


class UNet(nn.Module):
    def __init__(self, params, num_classes):
        super(UNet, self).__init__()
        # num_classes = params.dict['num_classes_bc']
        dropout_rate = params.dict["dropout_rate"]

        self.max_pool_2x2 = nn.MaxPool2d(kernel_size=2, stride=2)
        self.down_conv_1 = double_conv(1, 32, dropout_rate)
        self.down_conv_2 = double_conv(32, 64, dropout_rate)
        self.down_conv_3 = double_conv(64, 128, dropout_rate)
        self.down_conv_4 = double_conv(128, 256, dropout_rate)
        self.down_conv_5 = double_conv(256, 512, dropout_rate)

        self.up_trans_1 = nn.ConvTranspose2d(
            in_channels=512, out_channels=256, kernel_size=2, stride=2
        )

        self.up_conv_1 = double_conv(512, 256, dropout_rate)

        self.up_trans_2 = nn.ConvTranspose2d(
            in_channels=256, out_channels=128, kernel_size=2, stride=2
        )

        self.up_conv_2 = double_conv(256, 128, dropout_rate)

        self.up_trans_3 = nn.ConvTranspose2d(
            in_channels=128, out_channels=64, kernel_size=2, stride=2
        )

        self.up_conv_3 = double_conv(128, 64, dropout_rate)

        self.up_trans_4 = nn.ConvTranspose2d(
            in_channels=64, out_channels=32, kernel_size=2, stride=2
        )

        self.up_conv_4 = double_conv(64, 32, dropout_rate)

        self.out = nn.Conv2d(in_channels=32, out_channels=num_classes, kernel_size=1)

    def forward(self, image):

        x1 = self.down_conv_1(image)
        # after each convolution apply max_pooling
        p1 = self.max_pool_2x2(x1)

        x2 = self.down_conv_2(p1)
        p2 = self.max_pool_2x2(x2)

        x3 = self.down_conv_3(p2)
        p3 = self.max_pool_2x2(x3)

        x4 = self.down_conv_4(p3)
        p4 = self.max_pool_2x2(x4)

        x5 = self.down_conv_5(p4)

        # decoder
        u6 = self.up_trans_1(x5)
        x6 = self.up_conv_1(torch.cat([u6, x4], 1))

        u7 = self.up_trans_2(x6)
        x7 = self.up_conv_2(torch.cat([u7, x3], 1))

        u8 = self.up_trans_3(x7)
        x8 = self.up_conv_3(torch.cat([u8, x2], 1))

        u9 = self.up_trans_4(x8)
        x9 = self.up_conv_4(torch.cat([u9, x1], 1))

        x = self.out(x9)
        x = F.softmax(x, dim=1)
        return x


class BasicConv2d(nn.Module):
    def __init__(self, in_channels, out_channels, dropout, **kwargs):
        super(BasicConv2d, self).__init__()
        self.conv = nn.Conv2d(
            in_channels,
            out_channels,
            kernel_size=3,
            stride=1,
            padding="same",
            bias=True,
            **kwargs
        )
        # self.norm = nn.GroupNorm(32, out_channels, affine=True)
        # self.norm = nn.InstanceNorm2d(out_channels, affine=True)
        self.norm = nn.BatchNorm2d(out_channels)
        self.dropout = nn.Dropout(dropout)

    def forward(self, x):
        x = self.conv(x)
        x = self.norm(x)
        x = F.relu(x, inplace=True)
        x = self.dropout(x)
        return x


class AttentionGate2D(nn.Module):
    def __init__(self, in_channels_g, in_channels_l, out_channels, **kwargs):
        super(AttentionGate2D, self).__init__()
        self.wg = nn.Conv2d(
            in_channels_g, out_channels, bias=True, padding=0, kernel_size=1, stride=1
        )
        # self.norm_wg = nn.GroupNorm(num_groups=32, num_channels=out_channels, affine=True)
        # self.norm_wg = nn.InstanceNorm2d(out_channels, affine=True)
        self.norm_wg = nn.BatchNorm2d(out_channels)
        self.wx = nn.Conv2d(
            in_channels_l, out_channels, bias=True, padding=0, kernel_size=1, stride=1
        )
        # self.norm_wx = nn.GroupNorm(num_groups=32, num_channels=out_channels, affine=True)
        # self.norm_wx = nn.InstanceNorm2d(out_channels, affine=True)
        self.norm_wx = nn.BatchNorm2d(out_channels)
        self.psi = nn.Conv2d(
            in_channels=out_channels,
            out_channels=1,
            bias=True,
            padding=0,
            kernel_size=1,
            stride=1,
        )
        # self.norm_psi = nn.GroupNorm(num_groups=32, num_channels=out_channels, affine=True)
        # self.norm_psi = nn.InstanceNorm2d(out_channels, affine=True)
        self.norm_psi = nn.BatchNorm2d(1)

    def forward(self, g, x):
        g1 = self.wg(g)
        g1 = self.norm_wg(g1)
        x1 = self.wx(x)
        x1 = self.norm_wx(x1)
        psi = F.relu(g1 + x1, inplace=True)
        psi = self.psi(psi)
        psi = self.norm_psi(psi)
        psi = torch.sigmoid(psi)
        return x * psi


class ConvBlock2D(nn.Module):
    def __init__(self, in_channels, out_channels, dropout, **kwargs):
        super(ConvBlock2D, self).__init__()
        self.conv1 = BasicConv2d(
            in_channels=in_channels, out_channels=out_channels, dropout=dropout
        )
        self.conv2 = BasicConv2d(
            in_channels=out_channels, out_channels=out_channels, dropout=dropout
        )

    def forward(self, input):
        x1 = self.conv1(input)
        x2 = self.conv2(x1)
        return x2


class AttentionUNet(nn.Module):
    def __init__(self, params, num_classes):
        super(AttentionUNet, self).__init__()
        # num_classes = params.dict['num_classes_bc']
        dropout_rate = params.dict["dropout_rate"]

        self.max_pool_2x2 = nn.MaxPool2d(kernel_size=2, stride=2)
        # self.down_conv_1 = double_conv(1, 32, dropout_rate)
        self.down1 = ConvBlock2D(1, 32, dropout_rate)
        # self.down_conv_2 = double_conv(32, 64, dropout_rate)
        self.down2 = ConvBlock2D(32, 64, dropout_rate)
        # self.down_conv_3 = double_conv(64, 128, dropout_rate)
        self.down3 = ConvBlock2D(64, 128, dropout_rate)
        # self.down_conv_4 = double_conv(128, 256, dropout_rate)
        self.down4 = ConvBlock2D(128, 256, dropout_rate)
        # self.down_conv_5 = double_conv(256, 512, dropout_rate)
        self.down5 = ConvBlock2D(256, 512, dropout_rate)

        self.up_trans_5 = nn.ConvTranspose2d(
            in_channels=512, out_channels=256, kernel_size=2, stride=2
        )

        # self.up_conv_1 = double_conv(512, 256, dropout_rate)
        self.up5 = ConvBlock2D(512, 256, dropout_rate)
        self.ag5 = AttentionGate2D(256, 256, 128)

        self.up_trans_4 = nn.ConvTranspose2d(
            in_channels=256, out_channels=128, kernel_size=2, stride=2
        )

        # self.up_conv_2 = double_conv(256, 128, dropout_rate)
        self.up4 = ConvBlock2D(256, 128, dropout_rate)
        self.ag4 = AttentionGate2D(128, 128, 64)
        self.up_trans_3 = nn.ConvTranspose2d(
            in_channels=128, out_channels=64, kernel_size=2, stride=2
        )

        # self.up_conv_3 = double_conv(128, 64, dropout_rate)
        self.up3 = ConvBlock2D(128, 64, dropout_rate)
        self.ag3 = AttentionGate2D(64, 64, 32)

        self.up_trans_2 = nn.ConvTranspose2d(
            in_channels=64, out_channels=32, kernel_size=2, stride=2
        )

        # self.up_conv_4 = double_conv(64, 32, dropout_rate)
        self.up2 = ConvBlock2D(64, 32, dropout_rate)
        self.ag2 = AttentionGate2D(32, 32, 32)
        self.out = nn.Conv2d(in_channels=32, out_channels=num_classes, kernel_size=1)

    def forward(self, image):

        x1 = self.down1(image)
        # after each convolution apply max_pooling
        p1 = self.max_pool_2x2(x1)

        x2 = self.down2(p1)
        p2 = self.max_pool_2x2(x2)

        x3 = self.down3(p2)
        p3 = self.max_pool_2x2(x3)

        x4 = self.down4(p3)
        p4 = self.max_pool_2x2(x4)

        x5 = self.down5(p4)

        # decoder
        u6 = self.up_trans_5(x5)
        s4 = self.ag5(g=u6, x=x4)
        x6 = self.up5(torch.cat([u6, s4], 1))

        u7 = self.up_trans_4(x6)
        s3 = self.ag4(u7, x3)
        x7 = self.up4(torch.cat([u7, s3], 1))

        u8 = self.up_trans_3(x7)
        s2 = self.ag3(u8, x2)
        x8 = self.up3(torch.cat([u8, s2], 1))

        u9 = self.up_trans_2(x8)
        s1 = self.ag2(u9, x1)
        x9 = self.up2(torch.cat([u9, s1], 1))

        x = self.out(x9)
        x = F.softmax(x, dim=1)
        return x


if __name__ == "__main__":
    import pathlib

    from param_loader import Params

    param_path = (
        pathlib.Path(r"C:\Users\Winter\Documents\Projects\full-body-body-composition")
        / "params.json"
    )
    params = Params(param_path)
    img = torch.rand((4, 1, 512, 512))
    model = AttentionUNet(params, 2).to(device="cpu")
    summary(model, input_size=(1, 512, 512), device="cpu")
    outputs = model(img)
    print(outputs.size())
# %%



import os
import shutil
import constants


def find_l3_for_nifti_file(nifti_base_file_name):
    for root, dirs, files in os.walk(constants.LOCAL_L3_DIR):
        for f in files:
            if f == nifti_base_file_name + '.dcm':
                return os.path.join(root, f)
    return None


def main():
    for nifti_file_name in os.listdir(constants.LOCAL_SCAN_NIFTI_DIR):
        nifti_base_file_name = nifti_file_name[:-7]
        l3_file_path = find_l3_for_nifti_file(nifti_base_file_name)
        if l3_file_path:
            shutil.copy(l3_file_path, constants.LOCAL_SCAN_NIFTI_L3_DIR)


if __name__ == '__main__':
    main()


import numpy as np
import matplotlib.pyplot as plt
import pydicom


def load_ct_hu(dicom_path: str) -> np.ndarray:
    ds = pydicom.dcmread(dicom_path)
    img = ds.pixel_array.astype(np.float32)

    slope = float(getattr(ds, "RescaleSlope", 1.0))
    intercept = float(getattr(ds, "RescaleIntercept", 0.0))
    hu = img * slope + intercept
    return hu


def window_image(hu: np.ndarray, window: float, level: float) -> np.ndarray:
    lo = level - window / 2.0
    hi = level + window / 2.0
    return np.clip(hu, lo, hi)


def label_mask_to_rgba(mask: np.ndarray, alpha: float = 0.35) -> np.ndarray:
    """
    mask: (H,W) with labels {0,1,5,7}
    returns RGBA float image (H,W,4) in [0..1]
    """
    h, w = mask.shape
    rgba = np.zeros((h, w, 4), dtype=np.float32)

    # background=0 => already transparent

    # muscle=1 => red
    m = (mask == 1)
    rgba[m] = (1.0, 0.0, 0.0, alpha)

    # visceral fat=5 => yellow
    v = (mask == 5)
    rgba[v] = (1.0, 1.0, 0.0, alpha)

    # subcutaneous fat=7 => cyan
    s = (mask == 7)
    rgba[s] = (0.0, 1.0, 1.0, alpha)

    return rgba


def show_ct_with_overlay(dicom_path: str, mask_npy_path: str,
                         window: float = 400, level: float = 40,
                         alpha: float = 0.35, flipud: bool = False):
    hu = load_ct_hu(dicom_path)
    mask = np.load(mask_npy_path)

    if mask.shape != hu.shape:
        raise ValueError(f"Shape mismatch: CT {hu.shape} vs mask {mask.shape}")

    print("Unique labels:", np.unique(mask).tolist())

    if flipud:
        hu = np.flipud(hu)
        mask = np.flipud(mask)

    hu_w = window_image(hu, window, level)
    overlay = label_mask_to_rgba(mask, alpha=alpha)

    plt.figure(figsize=(8, 8))
    plt.imshow(hu_w, cmap="gray")
    plt.imshow(overlay)  # RGBA overlay
    plt.axis("off")
    plt.title(f"CT (W/L={window}/{level}) + segmentation overlay")
    plt.show()


# Example usage:
show_ct_with_overlay(
    "D:\\Mosamatic\\MartijnBroen\\02-10-2025_twee_plaatjes\\org\\419474.dcm", 
    "D:\\Mosamatic\\MartijnBroen\\02-10-2025_twee_plaatjes\\defaultpipeline\\segmentmusclefatl3tensorflowtask\\419474.dcm.seg.npy", 
    window=400, 
    level=50, 
    alpha=1
)



#!/usr/bin/env python3
"""
Interactive VTK resection-plane demo:

- Loads CT (.nii/.nii.gz) and a mask volume (e.g. liver).
- Shows the original mask surface (wireframe) + the "remaining" mask surface (solid).
- Lets you drag/rotate an interactive plane widget to simulate a resection plane.
- Updates remaining volume (mL) live.

Keys:
  f  -> flip kept side of the plane
  r  -> reset plane to image center (normal = +X)
  q/ESC -> quit
"""

import argparse
import vtk
from vtk.util.numpy_support import vtk_to_numpy


def read_nifti(path: str) -> vtk.vtkImageData:
    r = vtk.vtkNIFTIImageReader()
    r.SetFileName(path)
    r.Update()
    return r.GetOutput()


def binarize_mask(img: vtk.vtkImageData, threshold: float = 0.5) -> vtk.vtkImageData:
    t = vtk.vtkImageThreshold()
    t.SetInputData(img)
    t.ThresholdByLower(threshold)  # values >= threshold -> InValue
    t.SetInValue(1)
    t.SetOutValue(0)
    t.SetOutputScalarTypeToUnsignedChar()
    t.Update()
    return t.GetOutput()


def count_ones(mask01: vtk.vtkImageData) -> int:
    """
    Counts voxels == 1 via a tiny histogram (fast, avoids numpy copy of full volume).
    Requires mask values only in {0,1}.
    """
    acc = vtk.vtkImageAccumulate()
    acc.SetInputData(mask01)
    acc.SetComponentExtent(0, 1, 0, 0, 0, 0)   # histogram bins for 0 and 1
    acc.SetComponentOrigin(0, 0, 0)
    acc.SetComponentSpacing(1, 1, 1)
    acc.Update()

    hist = acc.GetOutput().GetPointData().GetScalars()
    counts = vtk_to_numpy(hist)  # length 2
    return int(counts[1]) if counts.size >= 2 else 0


def volume_ml_from_count(mask01: vtk.vtkImageData, ones: int) -> float:
    sx, sy, sz = mask01.GetSpacing()  # typically mm
    voxel_vol_mm3 = float(sx * sy * sz)
    return (ones * voxel_vol_mm3) / 1000.0  # 1 mL = 1000 mm^3


def make_surface_actor(mask01: vtk.vtkImageData, iso: float = 0.5) -> tuple[vtk.vtkActor, vtk.vtkAlgorithm]:
    """
    Returns (actor, surface_extractor_filter)
    """
    # FlyingEdges3D is generally faster than MarchingCubes.
    surf = vtk.vtkFlyingEdges3D()
    surf.SetInputData(mask01)
    surf.SetValue(0, iso)
    surf.ComputeNormalsOn()
    surf.Update()

    mapper = vtk.vtkPolyDataMapper()
    mapper.SetInputConnection(surf.GetOutputPort())
    mapper.ScalarVisibilityOff()

    actor = vtk.vtkActor()
    actor.SetMapper(mapper)
    return actor, surf


def main():
    ct = read_nifti("D:\\Mosamatic\\TotalSegmentator\\patient1.nii.gz")
    mask = read_nifti("D:\\Mosamatic\\TotalSegmentator\\output\\liver.nii.gz")
    mask01 = binarize_mask(mask, threshold=0.5)

    # Precompute total volume once
    total_ones = count_ones(mask01)
    total_ml = volume_ml_from_count(mask01, total_ones)

    # --- Clipping pipeline (mask01 -> stencil -> clipped_mask01) ---
    keep_negative_halfspace = {"value": True}  # mutable for callbacks

    clip_plane = vtk.vtkPlane()  # implicit function used by stencil
    bounds = mask01.GetBounds()
    cx = 0.5 * (bounds[0] + bounds[1])
    cy = 0.5 * (bounds[2] + bounds[3])
    cz = 0.5 * (bounds[4] + bounds[5])
    clip_plane.SetOrigin(cx, cy, cz)
    clip_plane.SetNormal(1, 0, 0)

    to_stencil = vtk.vtkImplicitFunctionToImageStencil()
    to_stencil.SetInput(clip_plane)
    to_stencil.SetOutputOrigin(mask01.GetOrigin())
    to_stencil.SetOutputSpacing(mask01.GetSpacing())
    to_stencil.SetOutputWholeExtent(mask01.GetExtent())
    to_stencil.Update()

    stencil = vtk.vtkImageStencil()
    stencil.SetInputData(mask01)
    stencil.SetStencilConnection(to_stencil.GetOutputPort())
    stencil.SetBackgroundValue(0)
    stencil.ReverseStencilOff()  # keep plane(x) <= 0 by default
    stencil.Update()

    clipped_mask01 = stencil.GetOutput()

    # --- Actors (original + remaining) ---
    original_actor, _orig_surf = make_surface_actor(mask01)
    original_actor.GetProperty().SetRepresentationToWireframe()
    original_actor.GetProperty().SetLineWidth(2.0)
    original_actor.GetProperty().SetOpacity(0.35)

    remaining_actor = vtk.vtkActor()
    remaining_surf = vtk.vtkFlyingEdges3D()
    remaining_surf.SetInputConnection(stencil.GetOutputPort())  # live connection
    remaining_surf.SetValue(0, 0.5)
    remaining_surf.ComputeNormalsOn()
    remaining_surf.Update()

    remaining_mapper = vtk.vtkPolyDataMapper()
    remaining_mapper.SetInputConnection(remaining_surf.GetOutputPort())
    remaining_mapper.ScalarVisibilityOff()

    remaining_actor.SetMapper(remaining_mapper)
    remaining_actor.GetProperty().SetOpacity(1.0)

    # --- Text overlay ---
    text = vtk.vtkTextActor()
    text.GetTextProperty().SetFontSize(18)
    text.GetTextProperty().SetColor(1, 1, 1)
    text.SetDisplayPosition(15, 15)

    def update_text():
        rem_ones = count_ones(clipped_mask01)
        rem_ml = volume_ml_from_count(mask01, rem_ones)
        res_ml = max(total_ml - rem_ml, 0.0)
        side = "keep plane(x) <= 0" if keep_negative_halfspace["value"] else "keep plane(x) > 0"
        text.SetInput(
            f"Total: {total_ml:.1f} mL\n"
            f"Remaining: {rem_ml:.1f} mL\n"
            f"Resected: {res_ml:.1f} mL\n"
            f"[f] flip side ({side})"
        )

    update_text()

    # --- Renderer / window / interactor ---
    ren = vtk.vtkRenderer()
    ren.AddActor(original_actor)
    ren.AddActor(remaining_actor)
    ren.AddActor2D(text)
    ren.SetBackground(0.08, 0.09, 0.11)

    rw = vtk.vtkRenderWindow()
    rw.SetWindowName("VTK Resection Plane Volume Demo")
    rw.SetSize(1200, 800)
    rw.AddRenderer(ren)

    iren = vtk.vtkRenderWindowInteractor()
    iren.SetRenderWindow(rw)

    style = vtk.vtkInteractorStyleTrackballCamera()
    iren.SetInteractorStyle(style)

    # --- Interactive plane widget ---
    rep = vtk.vtkImplicitPlaneRepresentation()
    rep.SetPlaceFactor(1.0)
    rep.PlaceWidget(bounds)
    rep.SetNormal(1, 0, 0)
    rep.SetOrigin(cx, cy, cz)

    plane_widget = vtk.vtkImplicitPlaneWidget2()
    plane_widget.SetInteractor(iren)
    plane_widget.SetRepresentation(rep)
    plane_widget.SetEnabled(1)

    def apply_plane_from_widget():
        # Pull current plane from the widget and copy into our implicit function
        p = vtk.vtkPlane()
        rep.GetPlane(p)
        clip_plane.SetOrigin(p.GetOrigin())
        clip_plane.SetNormal(p.GetNormal())

        # Update which side we keep
        if keep_negative_halfspace["value"]:
            stencil.ReverseStencilOff()  # keep implicit<=0
        else:
            stencil.ReverseStencilOn()   # keep implicit>0

        # Force pipeline refresh
        to_stencil.Modified()
        stencil.Modified()
        stencil.Update()

        update_text()
        rw.Render()

    def on_plane_interaction(_obj, _evt):
        apply_plane_from_widget()

    plane_widget.AddObserver(vtk.vtkCommand.InteractionEvent, on_plane_interaction)
    plane_widget.AddObserver(vtk.vtkCommand.EndInteractionEvent, on_plane_interaction)

    # --- Key bindings ---
    def on_keypress(_obj, _evt):
        key = iren.GetKeySym().lower()
        if key == "f":
            keep_negative_halfspace["value"] = not keep_negative_halfspace["value"]
            apply_plane_from_widget()
        elif key == "r":
            rep.SetOrigin(cx, cy, cz)
            rep.SetNormal(1, 0, 0)
            apply_plane_from_widget()

    iren.AddObserver("KeyPressEvent", on_keypress)

    # Camera setup
    ren.ResetCamera()
    rw.Render()
    iren.Initialize()
    iren.Start()


if __name__ == "__main__":
    main()



from moosez import moose

INPUT_NIFTI_FILE = 'D:\\Mosamatic\\NicoleHildebrand\\24-04-2025_coronale_CT\\MUMC004_3_coronal_to_axial.nii.gz'
OUTPUT_DIR = 'D:\\Mosamatic\\NicoleHildebrand\\24-04-2025_coronale_CT\\moose_output'


def main():
    # For this dataset, no muscles can be found
    moose(
        INPUT_NIFTI_FILE, 
        model_names=['clin_ct_vertebrae', 'clin_ct_muscles'], 
        output_dir=OUTPUT_DIR, 
        accelerator=None,
    )


if  __name__ == '__main__':
    main()


import os
import pandas as pd

DB_IKNL = "D:\\Data\\EvaClaasens\\IKNL dataset 2017-2022 KWF retrospectief 02.05.2024.xlsx"
DB_PALG = "D:\\Data\\EvaClaasens\\PALGA_retrospective 2017-2022.xlsx"
DB_INFO = "D:\\Data\\EvaClaasens\\output.xlsx"


def main():
    df_iknl = pd.read_excel(DB_IKNL, index_col='key_nkr')
    df_palg = pd.read_excel(DB_PALG, index_col='KEY_NKR')
    data = {
        'key_nkr': [],
        'excerpt_nrs': [],
    }
    for idx1, row1 in df_iknl.iterrows():
        key_nkr = idx1
        dcisbiopt = row1['dcisbiopt']
        if dcisbiopt == 1.0:
            excerpt_nrs = ''
            for idx2, row2 in df_palg.iterrows():
                if idx2 == key_nkr:
                    palgaexcerptnr = str(row2['PALGAexcerptnr'])
                    excerpt_nrs += palgaexcerptnr + '/'
            data['key_nkr'].append(key_nkr)
            data['excerpt_nrs'].append(excerpt_nrs)
            print(key_nkr)
    df = pd.DataFrame(data=data)
    df.to_excel(DB_INFO, index=False, engine='openpyxl')


if __name__ == '__main__':
    main()


import json
import pandas as pd
import matplotlib.pyplot as plt
from scipy.stats import ttest_rel, linregress

BC_SCORES_FILE_PATH = 'M:\\data\\contrastsensitivity\\lieke\\output\\defaultpipeline\\calculatescorestask\\bc_scores.csv'
COEFFICIENT_FILE_PATH = 'M:\\data\\contrastsensitivity\\lieke\\output\\coefficients.json'

def load_data():
    df = pd.read_csv(BC_SCORES_FILE_PATH, sep=';')
    df = df.assign(patient_id=df['file'].str.split('_').str[0])
    return df

def load_scores_unenhanced(df):
    return df[df["file"].str.contains("unenhanced", case=False, na=False)].copy()

def load_scores_arterial(df):
    return df[df["file"].str.contains("arterial", case=False, na=False)].copy()

def load_scores_venous(df):
    return df[df["file"].str.contains("venous", case=False, na=False)].copy()

def align_on_common_patients(*dfs):
    common = set(dfs[0]["patient_id"])
    for d in dfs[1:]:
        common &= set(d["patient_id"])
    return [d[d["patient_id"].isin(common)].copy() for d in dfs], common

def ttest(df1, df2, column):
    t_stat, p_value = ttest_rel(df1[column], df2[column], nan_policy='omit')
    print(f'[{column}] t = {t_stat:.3f}, p-value = {p_value:.3g}')
    return t_stat, p_value

def print_ttest_results(df_venous, df_unenhanced, df_arterial):
    print('\nUnenhanced vs. venous:')
    ttest(df_unenhanced, df_venous, column='muscle_ra')
    ttest(df_unenhanced, df_venous, column='muscle_area')
    ttest(df_unenhanced, df_venous, column='vat_area')
    ttest(df_unenhanced, df_venous, column='vat_ra')
    ttest(df_unenhanced, df_venous, column='sat_area')
    ttest(df_unenhanced, df_venous, column='sat_ra')
    print('\nArterial vs venous:')
    ttest(df_arterial, df_venous, column='muscle_ra')
    ttest(df_arterial, df_venous, column='muscle_area')
    ttest(df_arterial, df_venous, column='vat_area')
    ttest(df_arterial, df_venous, column='vat_ra')
    ttest(df_arterial, df_venous, column='sat_area')
    ttest(df_arterial, df_venous, column='sat_ra')

def create_scatter_plots(df_venous, df_unenhanced, df_arterial):
    fig = plt.figure(figsize=(30, 15), layout='constrained')
    subfigs = fig.subfigures(nrows=1, ncols=2)
    subfigs[0].suptitle('Unenhanced vs. venous')
    axes0 = subfigs[0].subplots(3, 2, sharex=False, sharey=False)
    axes0[0, 0].scatter(x=df_unenhanced['muscle_area'], y=df_venous['muscle_area'])
    axes0[0, 0].set_xlabel('Unenhanced')
    axes0[0, 0].set_ylabel('Venous')
    axes0[0, 0].set_title('Muscle area')
    axes0[0, 1].scatter(x=df_unenhanced['muscle_ra'], y=df_venous['muscle_ra'])
    axes0[0, 1].set_xlabel('Unenhanced')
    axes0[0, 1].set_ylabel('Venous')
    axes0[0, 1].set_title('Muscle RA')
    axes0[1, 0].scatter(x=df_unenhanced['vat_area'], y=df_venous['vat_area'])
    axes0[1, 0].set_xlabel('Unenhanced')
    axes0[1, 0].set_ylabel('Venous')
    axes0[1, 0].set_title('VAT area')
    axes0[1, 1].scatter(x=df_unenhanced['vat_ra'], y=df_venous['vat_ra'])
    axes0[1, 1].set_xlabel('Unenhanced')
    axes0[1, 1].set_ylabel('Venous')
    axes0[1, 1].set_title('VAT RA')
    axes0[2, 0].scatter(x=df_unenhanced['sat_area'], y=df_venous['sat_area'])
    axes0[2, 0].set_xlabel('Unenhanced')
    axes0[2, 0].set_ylabel('Venous')
    axes0[2, 0].set_title('SAT area')
    axes0[2, 1].scatter(x=df_unenhanced['sat_ra'], y=df_venous['sat_ra'])
    axes0[2, 1].set_xlabel('Unenhanced')
    axes0[2, 1].set_ylabel('Venous')
    axes0[2, 1].set_title('SAT RA')
    subfigs[1].suptitle('Arterial vs. venous')
    axes1 = subfigs[1].subplots(3, 2, sharex=False, sharey=False)
    axes1[0, 0].scatter(x=df_arterial['muscle_area'], y=df_venous['muscle_area'])
    axes1[0, 0].set_xlabel('Arterial')
    axes1[0, 0].set_ylabel('Venous')
    axes1[0, 0].set_title('Muscle area')
    axes1[0, 1].scatter(x=df_arterial['muscle_ra'], y=df_venous['muscle_ra'])
    axes1[0, 1].set_xlabel('Arterial')
    axes1[0, 1].set_ylabel('Venous')
    axes1[0, 1].set_title('Muscle RA')
    axes1[1, 0].scatter(x=df_arterial['vat_area'], y=df_venous['vat_area'])
    axes1[1, 0].set_xlabel('Arterial')
    axes1[1, 0].set_ylabel('Venous')
    axes1[1, 0].set_title('VAT area')
    axes1[1, 1].scatter(x=df_arterial['vat_ra'], y=df_venous['vat_ra'])
    axes1[1, 1].set_xlabel('Arterial')
    axes1[1, 1].set_ylabel('Venous')
    axes1[1, 1].set_title('VAT RA')
    axes1[2, 0].scatter(x=df_arterial['sat_area'], y=df_venous['sat_area'])
    axes1[2, 0].set_xlabel('Arterial')
    axes1[2, 0].set_ylabel('Venous')
    axes1[2, 0].set_title('SAT area')
    axes1[2, 1].scatter(x=df_arterial['sat_ra'], y=df_venous['sat_ra'])
    axes1[2, 1].set_xlabel('Arterial')
    axes1[2, 1].set_ylabel('Venous')
    axes1[2, 1].set_title('SAT RA')
    plt.show()

def create_regression_coefficients(df_venous, df_unenhanced, df_arterial):
    coefficients = {
        'unenhanced_to_venous': {
            'muscle_area': {'slope': 0, 'intercept': 0},
            'muscle_ra': {'slope': 0, 'intercept': 0},
            'vat_area': {'slope': 0, 'intercept': 0},
            'vat_ra': {'slope': 0, 'intercept': 0},
            'sat_area': {'slope': 0, 'intercept': 0},
            'sat_ra': {'slope': 0, 'intercept': 0},
        },
        'arterial_to_venous': {
            'muscle_area': {'slope': 0, 'intercept': 0},
            'muscle_ra': {'slope': 0, 'intercept': 0},
            'vat_area': {'slope': 0, 'intercept': 0},
            'vat_ra': {'slope': 0, 'intercept': 0},
            'sat_area': {'slope': 0, 'intercept': 0},
            'sat_ra': {'slope': 0, 'intercept': 0},
        }
    }
    for column in ['muscle_area', 'muscle_ra', 'vat_area', 'vat_ra', 'sat_area', 'sat_ra']:
        result = linregress(x=df_unenhanced[column].to_numpy(), y=df_venous[column].to_numpy())
        coefficients['unenhanced_to_venous'][column]['slope'] = result.slope
        coefficients['unenhanced_to_venous'][column]['intercept'] = result.intercept
    for column in ['muscle_area', 'muscle_ra', 'vat_area', 'vat_ra', 'sat_area', 'sat_ra']:
        result = linregress(x=df_arterial[column].to_numpy(), y=df_venous[column].to_numpy())
        coefficients['arterial_to_venous'][column]['slope'] = result.slope
        coefficients['arterial_to_venous'][column]['intercept'] = result.intercept
    with open(COEFFICIENT_FILE_PATH, 'w') as f:
        json.dump(coefficients, f, indent=4)
    return coefficients

def transform_df_to_venous(df_old, coefficients):
    df_new = df_old.copy()
    for column in ['muscle_area', 'muscle_ra', 'vat_area', 'vat_ra', 'sat_area', 'sat_ra']:
        slope = coefficients[column]['slope']
        intercept = coefficients[column]['intercept']
        df_new[column] = slope * df_new[column].to_numpy() + intercept
    return df_new

def main():
    # Load BC scores, determine smallest df and align all dfs 
    # on common patients
    df = load_data()
    df_venous = load_scores_venous(df)
    df_smallest = df_venous
    df_unenhanced = load_scores_unenhanced(df)
    if df_unenhanced.shape[0] < df_smallest.shape[0]:
        df_smallest = df_unenhanced
    df_arterial = load_scores_arterial(df)
    if df_arterial.shape[0] < df_smallest.shape[0]:
        df_smallest = df_arterial
    (dfs_aligned, common_ids) = align_on_common_patients(df_venous, df_unenhanced, df_arterial)
    df_venous, df_unenhanced, df_arterial = dfs_aligned
    print_ttest_results(
        df_venous, df_unenhanced, df_arterial)
    coefficients = create_regression_coefficients(df_venous, df_unenhanced, df_arterial)
    df_unenhanced_new = transform_df_to_venous(df_unenhanced, coefficients['unenhanced_to_venous'])
    df_arterial_new = transform_df_to_venous(df_arterial, coefficients['arterial_to_venous'])
    print_ttest_results(
        df_venous, df_unenhanced_new, df_arterial_new)
    # create_scatter_plots(df_venous, df_unenhanced, df_arterial)

if __name__ == '__main__':
    main()


def is_docker_running():
    import docker
    try:
        client = docker.from_env()
        client.ping()
        return True
    except Exception:
        return False


print(is_docker_running())


import os
import numpy as np
from PySide6.QtCore import Qt
from PySide6.QtWidgets import (
    QSplitter,
    QHBoxLayout,
    QFileDialog,
)
from mosamatic2.core.managers.logmanager import LogManager
from mosamatic2.core.data.dicomloader import DicomLoader
from mosamatic2.ui.widgets.panels.defaultpanel import DefaultPanel
from mosamatic2.ui.widgets.panels.tools.segmentationeditor.segmentationeditorcontrols import SegmentationEditorControls
from mosamatic2.ui.widgets.panels.tools.segmentationeditor.segmentationeditorgraphicsview import SegmentationEditorGraphicsView
from mosamatic2.ui.widgets.panels.tools.segmentationeditor.document import Document

LOG = LogManager()
PANEL_TITLE = 'SegmentationEditor'
PANEL_NAME = 'segmentationeditor'


class SegmentationEditor(DefaultPanel):

    #-------------------------------------------------------------------------------------------------------
    def __init__(self, settings):
        super(SegmentationEditor, self).__init__()
        self.set_title(PANEL_TITLE)
        self._view = None
        self._controls = None
        self.init()

    # INITIALIZATION

    #-------------------------------------------------------------------------------------------------------
    def init(self):
        splitter = QSplitter(Qt.Horizontal)
        splitter.addWidget(self.view())
        splitter.addWidget(self.controls())
        splitter.setChildrenCollapsible(False)
        splitter.setStretchFactor(0, 1)
        splitter.setStretchFactor(1, 0)
        layout = QHBoxLayout()
        layout.addWidget(splitter)
        self.setLayout(layout)
        self.setObjectName(PANEL_NAME)

    # GETTERS

    #-------------------------------------------------------------------------------------------------------
    def view(self):
        if not self._view:
            self._view = SegmentationEditorGraphicsView(
                self.controls().active_label(),
                self.controls().brush_radius(),
                self.controls().smart_paint(),
                self.controls().fix_non_active_labels(),
                self.controls().overall_opacity(),
                self.controls().hu_lo_muscle(),
                self.controls().hu_hi_muscle(),
                self.controls().hu_lo_vat(),
                self.controls().hu_hi_vat(),
                self.controls().hu_lo_sat(),
                self.controls().hu_hi_sat(),
                self,
            )
            self._view.active_label_changed.connect(self.controls().set_active_label)
            self._view.brush_radius_changed.connect(self.controls().set_brush_radius)
            self._view.smart_paint_changed.connect(self.controls().set_smart_paint)
            self._view.fix_non_active_labels_changed.connect(self.controls().set_fix_non_active_labels)
        return self._view
    
    #-------------------------------------------------------------------------------------------------------
    def controls(self):
        if not self._controls:
            self._controls = SegmentationEditorControls(
                self.settings().get_int('annotationview/active_label', 1),
                self.settings().get_int('annotationview/brush_radius', 10),
                self.settings().get_bool('annotationview/smart_paint', True),
                self.settings().get_bool('annotationview/fix_non_active_labels', True),
                self.settings().get_float('annotationview/overall_opacity', 0.5),
                self.settings().get_int('annotationview/hu_lo_muscle', -29),
                self.settings().get_int('annotationview/hu_hi_muscle', 150),
                self.settings().get_int('annotationview/hu_lo_vat', -150),
                self.settings().get_int('annotationview/hu_hi_vat', -50),
                self.settings().get_int('annotationview/hu_lo_sat', -190),
                self.settings().get_int('annotationview/hu_hi_sat', -30),
                self,
            )
            self._controls.image_loaded.connect(self.handle_image_loaded)
            self._controls.segmentation_loaded.connect(self.handle_segmentation_loaded)
            self._controls.segmentation_saved.connect(self.handle_segmentation_saved)
            self._controls.active_label_changed.connect(self.handle_active_label_changed)
            self._controls.smart_paint_changed.connect(self.handle_smart_paint_changed)
            self._controls.fix_non_active_labels_changed.connect(self.handle_fix_non_active_labels_changed)
            self._controls.hu_lo_muscle_changed.connect(self.handle_hu_lo_range_muscle_changed)
            self._controls.hu_hi_muscle_changed.connect(self.handle_hu_hi_range_muscle_changed)
            self._controls.hu_lo_vat_changed.connect(self.handle_hu_lo_range_vat_changed)
            self._controls.hu_hi_vat_changed.connect(self.handle_hu_hi_range_vat_changed)
            self._controls.hu_lo_sat_changed.connect(self.handle_hu_lo_range_sat_changed)
            self._controls.hu_hi_sat_changed.connect(self.handle_hu_hi_range_sat_changed)
            self._controls.overall_opacity_changed.connect(self.handle_overall_opacity_changed)
            self._controls.clear_labels.connect(self.handle_clear_labels)
            self._controls.reset_zoom.connect(self.handle_reset_zoom)
        return self._controls
    
    # EVENT HANDLERS

    #-------------------------------------------------------------------------------------------------------
    def handle_image_loaded(self):
        last_directory = self.settings().get('last_directory', None)
        file_path, _ = QFileDialog.getOpenFileName(dir=last_directory)
        if file_path:
            loader = DicomLoader(file_path)
            img, disp8, meta = loader.load_as_numpy_and_u8disp()
            if img is not None and disp8 is not None and meta is not None:
                mask = None
                seg_path = file_path + '.seg.npy'
                if os.path.isfile(seg_path):
                    mask = np.load(seg_path).astype(np.uint8)
                seg_path = file_path + '.npz'
                if os.path.isfile(seg_path):
                    data = np.load(file_path, allow_pickle=True)
                    mask = data["mask"].astype(np.uint8)
                if mask is None:
                    mask = np.zeros_like(disp8, dtype=np.uint8)
                self.view().set_document(Document(img=img, disp8=disp8, mask=mask, meta=meta))
                self.settings().set('last_directory', os.path.split(file_path)[0])

    #-------------------------------------------------------------------------------------------------------
    def handle_segmentation_loaded(self):
        pass

    #-------------------------------------------------------------------------------------------------------
    def handle_segmentation_saved(self):
        # TODO: implement this!!!
        pass

    #-------------------------------------------------------------------------------------------------------
    def handle_active_label_changed(self, label):
        self.view().set_active_label(label)

    #-------------------------------------------------------------------------------------------------------
    def handle_smart_paint_changed(self, smart_paint):
        self.view().set_smart_paint(smart_paint)

    #-------------------------------------------------------------------------------------------------------
    def handle_fix_non_active_labels_changed(self, fix_non_active_labels):
        self.view().set_fix_non_active_labels(fix_non_active_labels)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_lo_range_muscle_changed(self, value):
        self.view().set_hu_lo_muscle(value)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_hi_range_muscle_changed(self, value):
        self.view().set_hu_hi_muscle(value)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_lo_range_vat_changed(self, value):
        self.view().set_hu_lo_vat(value)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_hi_range_vat_changed(self, value):
        self.view().set_hu_hi_vat(value)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_lo_range_sat_changed(self, value):
        self.view().set_hu_lo_sat(value)

    #-------------------------------------------------------------------------------------------------------
    def handle_hu_hi_range_sat_changed(self, value):
        self.view().set_hu_hi_sat(value)

    #-------------------------------------------------------------------------------------------------------
    def handle_overall_opacity_changed(self, value):
        self.view().set_overall_opacity(value)

    #-------------------------------------------------------------------------------------------------------
    def handle_clear_labels(self):
        self.view().clear_labels()

    #-------------------------------------------------------------------------------------------------------
    def handle_reset_zoom(self):
        self.view().reset_zoom()